'*************************************************************************************************
'
' DEPENDENCIES: NONE
'
'    DATE: 09/03/2010 - Ludvik Jerabek - Initial Release
' Version: 1.0.1
' Comment: Script is designed to be modified to suit needs
' License: CPOL
'
'
' Revisions:
'
' 09/03/2010 - Ludvik Jerabek - Initial Release
' 09/03/2010 - Ludvik Jerabek - Added match Function 
' 09/03/2010 - Ludvik Jerabek - Added getosver Function
' 09/03/2010 - Ludvik Jerabek - Added nslookup function
' 09/03/2010 - Ludvik Jerabek - Added ping function
' 09/03/2010 - Ludvik Jerabek - Added rshift bitwise shift right operator
' 09/03/2010 - Ludvik Jerabek - Added lshift bitwise shift left operator
' 09/03/2010 - Ludvik Jerabek - Added std_logfile for multi level logging
' 09/09/2010 - Ludvik Jerabek - Added std_registry to prevent provider connects
' 09/03/2010 - Ludvik Jerabek - Added std_mail for SMTP mail
' 09/03/2010 - Ludvik Jerabek - Added std_fs_search class added allows regex UNIX style file find
' 09/03/2010 - Ludvik Jerabek - Added std_process class added to help manage remote processes
' 09/03/2010 - Ludvik Jerabek - Added std_service class added to help manage remote services
' 09/03/2010 - Ludvik Jerabek - Added std_auto class to prevent having to use "set" 
' 09/03/2010 - Ludvik Jerabek - Added std_stack generic stack object
' 09/03/2010 - Ludvik Jerabek - Added std_queue generic queue object
' 09/03/2010 - Ludvik Jerabek - Added std_deque generic double ended queue
' 09/03/2010 - Ludvik Jerabek - Added std_rb_tree generic red black tree 
' 02/17/2011 - Ludvik Jerabek - Added std_iterator for std_queue, std_stack, std_deque, and std_rb_tree enumeration
' 02/18/2011 - Ludvik Jerabek - Added get_iterator to std_queue, std_stack, and std_rb_tree
' 02/18/2011 - Ludvik Jerabek - Added get_rev_iterator to std_deque
' 02/22/2011 - Ludvik Jerabek - Added std_rc4 basic encryption
' 03/24/2011 - Ludvik Jerabek - Added strftime for time formatting
' 03/25/2011 - Ludvik Jerabek - Enhanced std_logfile allows custom date format and custom logging format
' 02/16/2012 - Ludvik Jerabek - Added std_hostfile allows modification of windows hostfile
' 02/16/2012 - Ludvik Jerabek - Added std_logoninfo used for querying logon groups and shares
' 03/26/2013 - Ludvik Jerabek - Added ini_read and ini_write
'
'
' **DISCLAIMER**
' THIS MATERIAL IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
' EITHER EXPRESS OR IMPLIED, INCLUDING, BUT Not LIMITED TO, THE
' IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
' PURPOSE, OR NON-INFRINGEMENT. SOME JURISDICTIONS DO NOT ALLOW THE
' EXCLUSION OF IMPLIED WARRANTIES, SO THE ABOVE EXCLUSION MAY NOT
' APPLY TO YOU. IN NO EVENT WILL I BE LIABLE TO ANY PARTY FOR ANY
' DIRECT, INDIRECT, SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY
' USE OF THIS MATERIAL INCLUDING, WITHOUT LIMITATION, ANY LOST
' PROFITS, BUSINESS INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON
' YOUR INFORMATION HANDLING SYSTEM OR OTHERWISE, EVEN If WE ARE
' EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
'*************************************************************************************************
Option Explicit
'Turn this off when debugging
'On Error Resume Next

' ************************************************************************************************
' GLOBAL OBJECT (KEEP THEM AT A MINIMUM) DON'T EDIT THIS!
' ************************************************************************************************
Dim objFSO : Set objFSO = CreateObject("Scripting.FileSystemObject")
Dim objShell : Set objShell = CreateObject("WScript.Shell")
Dim objProcessEnv :	Set objProcessEnv = objShell.Environment("Process")

' SEE SECTION CALLED "LOG LEVEL" to set logging detail
Dim objLog : Set objLog = New std_logfile
Dim g_script_action

' ************************************************************************************************ 
' LOGFILE OUTPUT PATH 
' ************************************************************************************************

Const LOG_FILE = "C:\temp\script.log"
'Dim LOG_FILE : LOG_FILE = objShell.ExpandEnvironmentStrings("%SystemRoot%")&"\script.log"

' ************************************************************************************************ 
' LOGFILE CONSOLE OUTPUT
' ************************************************************************************************

' If verbose console output is needed this value can be changed to True
objLog.EnableOutputToConsole = False

' ************************************************************************************************
' !!!!                          TEMPLATE CONSTANTS DO NOT EDIT THEM                           !!!!
' ************************************************************************************************
' For Files Reads 
Const ForReading=1
Const ForWriting=2
Const ForAppending=8
Const TriStateAscii = 0
Const TriStateUnicode = -1
Const TriStateDefault = -2

' Service Types
Const SERVICE_TYPE_KDRIVER		= &H0001
Const SERVICE_TYPE_SDRIVER		= &H0002
Const SERVICE_TYPE_ADAPTER		= &H0004
Const SERVICE_TYPE_RDRIVER		= &H0008
Const SERVICE_TYPE_OWNPROC		= &H0010
Const SERVICE_TYPE_SHAREPROC	= &H0020
Const SERVICE_TYPE_INTERACT		= &H0100

' Logger Level Bits used by Class LogFile
Const LOG_SEVERE	= &H0020
Const LOG_ERROR		= &H0010
Const LOG_WARNING	= &H0008 
Const LOG_INFO		= &H0004
Const LOG_DEBUG		= &H0002
Const LOG_CONFIG	= &H0001
Const LOG_ALL		= &H003F 
Const LOG_OFF		= &H0000

' Service Error Control 
Const SERVICE_EC_SUPPRESS	= 0 'User is not notified.
Const SERVICE_EC_USER		= 1 'User is notified.
Const SERVICE_EC_LAST		= 2 'System is restarted with the last-known-good.
Const SERVICE_EC_GOOD		= 3 'System attempts to start with a good configuration.

' Service Start Modes
Const SERVICE_START_BOOT	= "Boot"
Const SERVICE_START_SYSTEM 	= "System"
Const SERVICE_START_AUTO	= "Automatic"
Const SERVICE_START_MANUAL	= "Manual"
Const SERVICE_START_DISABLE	= "Disabled"

' Service Errors
Const SERVICE_SUCCESS			= 0   'Success
Const SERVICE_NOT_SUPPORTED		= 1   'Not Supported
Const SERVICE_ACCESSD			= 2   'Access Denied
Const SERVICE_DEPEND			= 3   'Dependent Services Running
Const SERVICE_INVSC				= 4   'Invalid Service Control
Const SERVICE_SVC_NOACCCTR		= 5   'Service Cannot Accept Control
Const SERVICE_NOT_ACTIVE		= 6   'Service Not Active
Const SERVICE_TIMEOUT			= 7   'Service Request Timeout
Const SERVICE_UNKNOWN			= 8	  'Unknown Failure
Const SERVICE_PATH				= 9	  'Path Not Found
Const SERVICE_RUNNING			= 10  'Service Already Running
Const SERVICE_STATE_REQUESTED	= 10  'Results in the the requested state being true
Const SERVICE_DBLOCKED			= 11  'Service Database Locked
Const SERVICE_DEPDEL			= 12  'Service Dependency Deleted
Const SERVICE_DEPFAIL			= 13  'Service Dependency Failure
Const SERVICE_DISABLED			= 14  'Service Disabled
Const SERVICE_LOGIN_FAIL		= 15  'Service Logon Failure
Const SERVICE_MARKED_DEL		= 16  'Service Marked For Deletion
Const SERVICE_NOTHREAD			= 17  'Service No Thread
Const SERVICE_CIRCDEP			= 18  'Status Circular Dependency
Const SERVICE_DUPNAME			= 19  'Status Duplicate Name
Const SERVICE_INVNAME			= 20  'Status Invalid Name
Const SERVICE_INVPARAM			= 21  'Status Invalid Parameter
Const SERVICE_INVACCT			= 22  'Status Invalid Service Account
Const SERVICE_SVCEXISTS			= 23  'Status Service Exists
Const SERVICE_NOT_EXIST			= 24  'Service doesn't exists
Const SERVICE_WMI_FAIL			= 25  'Remote Service connect failed via WMI

' Process Errors
Const PROCESS_SUCCESS	= 0   'Success
Const PROCESS_ACCESSD	= 0   'Access Denied
Const PROCESS_UNKNOWN	= 8   'Unknown Error
Const PROCESS_PATH		= 9	  'Unknown Error
Const PROCESS_INVPARAM	= 21  'Invalid Parameter
Const PROCESS_EXISTS	= 23  'Process Exists
Const PROCESS_NOT_EXIST	= 24  'Process Not Exist
Const PROCESS_WMI_FAIL	= 25  'Remote Service connect failed via WMI

' WMI
Const WBEM_MAX_WAIT = &H80

' Service States
Const SERVICE_STAT_STOPPED	= "Stopped"
Const SERVICE_STAT_STOP_PENDING	= "Stop Pending"
Const SERVICE_STAT_RUNNING 	= "Running"
Const SERVICE_STAT_START_PENDING = "Start Pending"
Const SERVICE_STAT_PAUSED 	= "Paused"
Const SERVICE_STAT_PAUSE_PENDING = "Pause Pending"
Const SERVICE_STAT_CONTINUE_PENDING = "Continue Pending"
Const SERVICE_STAT_UNKNOWN = "Unknown"


' Registry Hives
Const HKEY_LOCAL_MACHINE 	= &H80000002
Const HKEY_CURRENT_USER 	= &H80000001
Const HKEY_CLASSES_ROOT 	= &H80000000
Const HKEY_USERS			= &H80000003
Const HKEY_CURRENT_CONFIG	= &H80000005
Const HKEY_DYN_DATA			= &H80000006

' Reg Value Types
Const REG_SZ		= 1
Const REG_EXPAND_SZ	= 2
Const REG_BINARY	= 3
Const REG_DWORD		= 4
Const REG_MULTI_SZ	= 7

' Registry Permissions
Const KEY_QUERY_VALUE			= &H00001
Const KEY_SET_VALUE				= &H00002
Const KEY_CREATE_SUB_KEY		= &H00004
Const KEY_ENUMERATE_SUB_KEYS	= &H00008
Const KEY_NOTIFY				= &H00016
Const KEY_CREATE				= &H00032
Const KEY_DELETE				= &H10000
Const KEY_READ_CONTROL			= &H20000
Const KEY_WRITE_DAC				= &H40000
Const KEY_WRITE_OWNER			= &H80000

' Attributes Generic (Standard File Attributes)
Const FS_ATTR_NRM = &H0000&	' Normal
Const FS_ATTR_RDO = &H0001&	' Readonly
Const FS_ATTR_HID = &H0002&	' Hidden
Const FS_ATTR_SYS = &H0004&	' System
Const FS_ATTR_VOL = &H0008&	' Volume
Const FS_ATTR_DIR = &H0010&	' Directory
Const FS_ATTR_ARC = &H0020&	' Archive
Const FS_ATTR_LNK = &H0400&	' Link
Const FS_ATTR_CMP = &H0800&	' Compressed
Const FS_ATTR_ALL = &H0C3F&	' All

' File Filters
Const FS_ATTR_FNRM = &H00000000& ' Normal
Const FS_ATTR_FRDO = &H00000001& ' Readonly
Const FS_ATTR_FHID = &H00000002& ' Hidden
Const FS_ATTR_FSYS = &H00000004& ' System
Const FS_ATTR_FVOL = &H00000008& ' Volume
Const FS_ATTR_FDIR = &H00000010& ' Directory
Const FS_ATTR_FARC = &H00000020& ' Archive
Const FS_ATTR_FLNK = &H00000400& ' Link
Const FS_ATTR_FCMP = &H00000800& ' Compressed
Const FS_ATTR_FANY = &H00000C3F& ' All
Const FS_ATTR_FSON = &H00008000& ' Enable File Search ( Must be passed to enable file search )

' Folder Filters
Const FS_ATTR_DNRM = &H00000000& ' Normal
Const FS_ATTR_DRDO = &H00010000& ' Readonly
Const FS_ATTR_DHID = &H00020000& ' Hidden
Const FS_ATTR_DSYS = &H00040000& ' System
Const FS_ATTR_DVOL = &H00080000& ' Volume
Const FS_ATTR_DDIR = &H00100000& ' Directory
Const FS_ATTR_DARC = &H00200000& ' Archive
Const FS_ATTR_DLNK = &H04000000& ' Link
Const FS_ATTR_DCMP = &H08000000& ' Compressed
Const FS_ATTR_DANY = &H0C3F0000& ' All
Const FS_ATTR_DSON = &H80000000& ' Enable Directory Search ( Must be passed to enable folder search )

' OS Masks
Const OS_TYPE_MASK	= &HF0 
Const OS_TYPE_SRV 	= &HF0
Const OS_TYPE_WKS 	= &H00

' OS Types Server Class 
Const OS_SRV_2K8R2		= &HF9
Const OS_SRV_2K8		= &HF8
Const OS_SRV_2K3R2		= &HF7
Const OS_SRV_2K3		= &HF6
Const OS_SRV_2K			= &HF5
Const OS_SRV_NT			= &HF4
Const OS_SRV_TOO_OLD 	= &HF3

' OS Types Workstation Class
Const OS_WKS_SEVEN		= &H0A
Const OS_WKS_VISTA		= &H09
Const OS_WKS_2K3		= &H08
Const OS_WKS_XP_X64		= &H07
Const OS_WKS_XP			= &H06
Const OS_WKS_2K			= &H05
Const OS_WKS_NT			= &H04
Const OS_WKS_TOO_OLD	= &H03
Const OS_FUNC_FAILED	= &H00

' SMTP Content Types
Const SMTP_CONTENT_HTML	= 1
Const SMTP_CONTENT_TEXT	= 2
Const SMTP_CONTENT_FILE	= 3

' Auth Type
Const SMTP_AUTH_BASIC	= 1
Const SMTP_AUTH_TEXT	= 2
Const SMTP_AUTH_NONE	= 3 

' ADODB Connection States
Const adStateClosed		= 0
Const adStateOpen		= 1
Const adStateConnecting	= 2
Const adStateExecuting	= 4
Const adStateFetching	= 8

' Windows Installer Flags
Const msiUILevelNoChange = 0
Const msiUILevelDefault = 1
Const msiUILevelNone = 2
Const msiUILevelBasic = 3
Const msiUILevelReduced = 4
Const msiUILevelFull = 5
Const msiUILevelHideCancel = 32
Const msiUILevelProgressOnly = 64
Const msiUILevelEndDialog = 128

Const msiInstallStateNotUsed      = -7
Const msiInstallStateBadConfig    = -6
Const msiInstallStateIncomplete   = -5
Const msiInstallStateSourceAbsent = -4
Const msiInstallStateInvalidArg   = -2
Const msiInstallStateUnknown      = -1
Const msiInstallStateBroken       =  0
Const msiInstallStateAdvertised   =  1
Const msiInstallStateRemoved      =  1
Const msiInstallStateAbsent       =  2
Const msiInstallStateLocal        =  3
Const msiInstallStateSource       =  4
Const msiInstallStateDefault      =  5

Const msiOpenDatabaseModeReadOnly		= 0 
Const msiOpenDatabaseModeTransact		= 1 
Const msiOpenDatabaseModeDirect			= 2 
Const msiOpenDatabaseModeCreate			= 3 
Const msiOpenDatabaseModeCreateDirect	= 4 
Const msiOpenDatabaseModeListScript		= 5 
Const msiOpenDatabaseModePatchFile		= 32 
 



' ************************************************************************************************ 
' LOG LEVEL   
' ************************************************************************************************
' Supported logger levels are listed below:
' LOG_SEVERE
' LOG_ERROR
' LOG_WARNING 
' LOG_INFO
' LOG_DEBUG
' LOG_CONFIG

' Logging level is any combination of these values or'ed to achive the propper output the
' Default logger level is LOG_ALL see the example below:

' EXAMPLE
' Logs all data which is classivied as SEVERE or a WARNING
' objLog.LoggerLevel = LOG_SEVERE or WARNING\

' Default Log Level
objLog.LoggerLevel = LOG_ALL


' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' TEMPLATE FUNCTIONS DO NOT MODIFY THEM!
' ************************************************************************************************
' START OF ALL TEMPLATE FUNCTIONS & CLASSES

' Used to import external code
Sub include( ByVal filename )
	Dim fso, f, fd 
	Set fso = CreateObject("Scripting.FileSystemObject") 
	Set f = fso.OpenTextFile( filename ) 
	fd = f.ReadAll() 
	f.close 
	ExecuteGlobal fd 
End Sub 

' Bit shift right
Function rshift( nInt , n ) RShift = (CLng(Abs(nInt)/CLng(2^n))) End Function
	
' Bit shift left
Function lshift( nInt , n ) LShift = (CLng(Abs(nInt)*CLng(2^n))) End Function 
		
' Hex String to Decimal max lenght is long int
Function hex2dec(strHex)
	Dim lngResult
	Dim intIndex
	Dim strDigit
	Dim intDigit
	Dim intValue
	lngResult = 0
	For intIndex = Len(strHex) To 1 Step -1
		strDigit = Mid(strHex, intIndex, 1)
		intDigit = InStr("0123456789ABCDEF", UCase(strDigit))-1
		If intDigit >= 0 Then
			intValue = intDigit * (16 ^ (Len(strHex)-intIndex))
			lngResult = CLng(lngResult + intValue)
		Else
			lngResult = 0
			intIndex = 0 ' stop the loop
		End If
	Next
	hex2dec = lngResult
End Function
		
'Recursive folder create, will create directories and Sub
Sub mkdir( ByVal strPath )
	If strPath <> "" And Not objFSO.FolderExists( strPath ) Then 'After the lowest possible folder is reached the string can be "" so you must stop...
		If Not objFSO.FolderExists( objFSO.GetParentFolderName(strPath) ) Then Call mkdir( objFSO.GetParentFolderName(strPath) )
		Call objFSO.CreateFolder( strPath )
	End If 
End Sub
		
'Returns a random index of an array based on the upper bound of the Array
Function rindex( arr )
	Randomize    
	rindex = CInt(UBound( arr ) * Rnd)
End Function 

' Class provide UNIX style file searching from VBScript
' Allows the user to search restricting depth, files types, and attributes		
Class std_fs_search
	Private Sub Class_Initialize()
		Set m_fnCallback = Nothing
		nDepth=1
	End Sub
	' Register callback function which returns results
	' Callback must be defined as SomeFunction( oFile , oData )
	' See the following example for usage:
	'
	' Function myCallBackFunc( oFile , oData )
	'     myCallBackFunc=False ' Keep searching
	'    'myCallBackFunc=True ' Stop searching
	'     WScript.Echo oFile.Path
	' End Function
	'
	' Dim fs : Set fs = New FileSearcher
	' Dim data
	' fs.RegisterCallback( GetRef("myCallBackFunc") )
	'
	Public Function RegisterCallback( byRef fnCallback )
		If Not IsObject( fnCallback ) Then Call Err.Raise( 424, "RegisterCallback", "Function expects an argument function pointer or Nothing" )
		Set m_fnCallback = fnCallback
	End Function
	
	' Searches for a file in strSearchBase
	' objSearchPrefs stores the search data
	' oData is any type which the user would like passed to the callback
	' function returns True only if the call back returns false which will abort the search
	' NOTE: See SearchPrefs class for information on search depth (default search depth is infinite)
	'       to prevent infinite recursion it is best set the SearchPref for depth to n levels of 
	'       recursion. Search depth can not be negative, 1 represents the strSearchBase.
	'       TypeName can be used to determine if oFile is a folder or a file to restrict
	'       by default SearchPref.Flags is set to (FS_FILES Or FS_FOLDERS) however it is
	'       possible to only have files sent to the call back by using the respective flag.
	Public Function Search( strSearchBase , byRef objSearchPrefs , byRef oData )
		Search=False		
		On Error Resume Next
		If objSearchPrefs.Depth <> 0 Then If objSearchPrefs.Depth < nDepth Then Exit Function 'Exit the function
		Dim file, folder, subFolder
		Set folder = objFSO.GetFolder(strSearchBase)
		If folder <> Empty Then
			If objSearchPrefs.Flags And FS_ATTR_FSON Then
				For Each file In folder.Files
					' Return match only if file attribute meets our search criteria
					If Match( file.Name , objSearchPrefs ) And (((objSearchPrefs.Flags And FS_ATTR_FANY) = FS_ATTR_FANY) Or (((objSearchPrefs.Flags And FS_ATTR_FANY) <> FS_ATTR_FANY) And ((file.Attributes And FS_ATTR_FANY) And (objSearchPrefs.Flags And FS_ATTR_FANY)))) Then
						If file <> Empty Then Search = m_fnCallback( file , oData )	
						If Eval(Search) Then Exit Function 'Exit the function
					End If
				Next
			End If
			If objSearchPrefs.Flags And FS_ATTR_DSON Then
				For Each subFolder In folder.SubFolders
					If Match( subFolder.Name , objSearchPrefs ) And (((objSearchPrefs.Flags And FS_ATTR_DANY) = FS_ATTR_DANY) Or (((objSearchPrefs.Flags And FS_ATTR_DANY) <> FS_ATTR_DANY) And ((folder.Attributes And RShift(FS_ATTR_DANY,16)) And RShift(objSearchPrefs.Flags And FS_ATTR_DANY,16)))) Then
						If subFolder <> Empty Then Search = m_fnCallback( subFolder , oData )
						If Eval(Search) Then Exit Function 'Exit the function
					End If
				Next
			End If
			' Recurse
			For Each subFolder In folder.SubFolders
				' Don't traverse links
				If (subFolder.Attributes And FS_ATTR_LNK) <> FS_ATTR_LNK Then
					' Fake recursive inc\dec
					nDepth = nDepth + 1
					Search = Search( subFolder , objSearchPrefs , oData )
					nDepth = nDepth - 1
					If Eval(Search) Then Exit Function 'Exit the function
				End If 
			Next
		End If
	End Function
	' Member Function Callback
	Private m_fnCallback
	Private nDepth
End Class
		
' Function tests a string to see if it matches a regular expression
' strToValidate - String to compare to the regex pattern
Function match( strToValidate , objSearchPrefs )
	If StrComp( TypeName(objSearchPrefs) , "std_searchprefs" , 1 ) = 0 Then ' Search with restriction	
		Dim rx : Set rx = New RegExp
		rx.Pattern = objSearchPrefs.SearchExp
		rx.Global = objSearchPrefs.Global
		rx.IgnoreCase = objSearchPrefs.NoCase
		rx.MultiLine = objSearchPrefs.MultiLine
		Match = rx.Test(strToValidate)
		Set rx = Nothing
	Else
		Call Err.Raise( 13 , "Match( strToValidate , objSearchPrefs )" , "objSearchPrefs should be an object of type SearchPrefs" ) 
	End If
End Function
		
' Function returns the operating system type information by ref variable OS_TYPE.
' The function will return true if the query for the OS type was a success. Or false if it was not.
Function osver( strHost , byRef strVersionText )
	On Error Resume Next
	osver = OS_FUNC_FAILED
	Dim objWMIService , colOperatingSystems , objOperatingSystem
	Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strHost & "\root\cimv2")
	If Not objWMIService Is Nothing Then
		Set colOperatingSystems = objWMIService.ExecQuery("Select * from Win32_OperatingSystem")
		For Each objOperatingSystem In colOperatingSystems
			Dim Version : Version = Split( objOperatingSystem.Version , "." )
			strVersionText = objOperatingSystem.Caption
			If objOperatingSystem.ProductType = 1 Then ' Workstation
				Select Case Version(0) & "." & Version(1)
					Case "6.1"
					osver = OS_WKS_SEVEN
					Case "6.0" 
					osver = OS_WKS_VISTA
					Case "5.2" 
					osver = OS_WKS_XP_X64
					Case "5.1" 
					osver = OS_WKS_XP
					Case "5.0" 
					osver = OS_WKS_2K
					Case "4.0" 
					osver = OS_WKS_NT
					Case Else
					osver = OS_WKS_TOO_OLD
				End Select
			Else ' Some kind of server product
				Select Case Version(0) & "." & Version(1)
					Case "6.1"
					osver = OS_SRV_2K8R2
					Case "6.0" 
					osver = OS_SRV_2K8
					Case "5.2" 
					If StrComp( objOperatingSystem.OtherTypeDescription , "R2" , 1  ) = 0 Then
						osver = OS_SRV_2K3R2
					Else
						osver = OS_SRV_2K3
					End If
					Case "5.0" 
					osver = OS_SRV_2K
					Case "4.0" 
					osver = OS_SRV_NT
					Case Else
					osver = OS_SRV_TOO_OLD
				End Select
			End If
		Next
	End If
	' Setting to Nothing is not needed, but we are doing it just to be sure
	Set objWMIService = Nothing
End Function
		
		
' Ping function will work on Windows 2000 and Windows XP without using the Win32_PingStatus
' Function pings a host and returns the True if the device is reachable or false if not
' if the return code is true then the ByRef arguments contain the results of the ping
Function ping(strHost , ByRef bytesSent , ByRef bytesReceived , ByRef bytesLost , ByRef minMs , ByRef maxMs , ByRef aveMs )
	ping = False
	Dim objShell, objExec, strPingResults, bRet
	Set objShell = CreateObject("WScript.Shell")
	Set objExec = objShell.Exec("ping -n 2 " & strHost)	
	Do
		WScript.Sleep 100
	Loop Until objExec.Status <> 0
	strPingResults = objExec.StdOut.ReadAll
	Dim regexpingstats : Set regexpingstats = New RegExp
	regexpingstats.Pattern = "Packets:\s+Sent\s+=\s+([0-9]+).*Received\s+=\s+([0-9]+).*Lost\s+=\s+([0-9]+)(?:.*\s)+Minimum\s+=\s+([0-9]+)ms.*Maximum\s+=\s+([0-9]+)ms.*Average\s+=\s+([0-9]+)ms"
	regexpingstats.Global = True
	regexpingstats.IgnoreCase = True
	regexpingstats.MultiLine = True
	If regexpingstats.Test(strPingResults) Then
		Dim m : Set m = regexpingstats.Execute(strPingResults)
		bytesSent = CInt(m.Item(0).subMatches.Item(0))
		bytesReceived = CInt(m.Item(0).subMatches.Item(1))
		bytesLost = CInt(m.Item(0).subMatches.Item(2))
		minMs = CInt(m.Item(0).subMatches.Item(3))
		maxMs = CInt(m.Item(0).subMatches.Item(4))
		aveMs = CInt(m.Item(0).subMatches.Item(5))
		ping = Eval( bytesSent > bytesLost )
	End If
End Function
		
' NSLookup Function
Function nslookup( strHostorIP , strNS , strNSIP , strName , ByRef arrIPs )
	nslookup = False
	Dim objShell, objExec, strNSResults , bRet, i , ips
	arrIPs = Array
	Set objShell = CreateObject("WScript.Shell")
	Set objExec = objShell.Exec("nslookup.exe " & strHostorIP)	
	Do
		WScript.Sleep 100
	Loop Until objExec.Status <> 0
	strNSResults = objExec.StdOut.ReadAll
	Dim regexpingstats : Set regexpingstats = New RegExp
	regexpingstats.Pattern = "Server:\s*(.*)\s*(.*\n)*Address:\s*(.*)\s*(.*\n)*Name:\s*(.*)\s*(.*\n)*(Address|Addresses):\s*(([0-9]{1,3}[.][0-9]{1,3}[.][0-9]{1,3}[.][0-9]{1,3}[\s,]*)+)"
	regexpingstats.Global = True
	regexpingstats.IgnoreCase = True
	regexpingstats.MultiLine = False
	If regexpingstats.Test(strNSResults) Then
		Dim m : Set m = regexpingstats.Execute(strNSResults)
		strNS = Replace( Replace( Replace( Replace( m.Item(0).subMatches.Item(0) , vbCr , "" ) , vbLf , "" ) , vbTab , "" ) , " " , "" )
		strNSIP = Replace( Replace( Replace( Replace( m.Item(0).subMatches.Item(2) , vbCr , "" ) , vbLf , "" ) , vbTab , "" ) , " " , "" )
		strName = Replace( Replace( Replace( Replace( m.Item(0).subMatches.Item(4) , vbCr , "" ) , vbLf , "" ) , vbTab , "" ) , " " , "" )
		ips = Replace( Replace( Replace( Replace( m.Item(0).subMatches.Item(7) , vbCr , "," ) , vbLf , "" ) , vbTab , "" ) , " " , "" )
		While Len( ips ) > 0 And InStrRev( ips , "," ) = Len( ips ) 
			ips = Mid( ips , 1 , InStrRev( ips , "," ) - 1 )
		Wend
		arrIPs = Split(ips,",")
		nslookup=True
	End If
End Function
		
' Returns Trus is OS supports 64bit code
Function isos64()
	isos64 = False
	Dim strWin : strWin = objFSO.GetSpecialFolder(0)
	If objFso.FolderExists(strWin & "\SysWOW64") Then isos64 = True
End Function

' Returns false if the script host is not cscript			
Function iscscript()
	' Check whether CScript.exe is the host.
	If (InStr(UCase(WScript.FullName), "CSCRIPT") <> 0) Then
		iscscript = True
	Else
		iscscript = False
	End If
End Function

' Returns a date string according to a pattern
' Eg. strftime( "%M/%D/%Y %I:%N:%S %P" , Now ) ' Will print 01/01/1970 12:00 AM
' %A - Full Weekday Name ( Monday , Tuesday )
' %a - Abbreviated weekday name ( Mon , Tue )
' %B - Full month Name ( January , February )
' %b - Abbreviated month name ( Jan , Feb )
' %Y	- 4 Digit Year
' %y	- 2 Digit Year
' %M	- 2 Digit Month (Padded 0)
' %m	- Digit Month
' %D 	- 2 Digit Day (Padded 0)
' %d 	- Digit Day
' %H 	- 2 Digit Hour 24hr (Padded 0)
' %h 	- Digit Hour 24hr
' %I 	- 2 Digit Hour 12hr (Padded 0)
' %i	- Digit Hour 12hr
' %N 	- 2 Digit Minute (Padded 0)
' %n 	- Digit Minute
' %S 	- 2 Digit Second (Padded 0)
' %s 	- Digit Second
' %q	- Quarter
' %J 	- Day of Year (Padded 0)
' %j 	- Day of Year (Padded 0)
' %w 	- Day of Week
' %U 	- Week of Year (Padded 0)
' %u 	- Week of Year
' %P 	- Meridiem (Upper Case)
' %p 	- Meridiem (Lower Case)
Function strftime( sFmt , oDate )
	strftime = sFmt
	Dim parse_patterns : parse_patterns = Array( "%Y","%y","%M","%m","%D","%d","%H","%h","%I","%i","%N","%n","%S","%s","%q","%J","%j","%w","%U","%u","%A","%a","%B","%b","%P","%p")
	Dim hour_twelve : hour_twelve = DatePart("h",oDate) Mod 12
	Dim meridiem : meridiem = Array( "AM" , "PM" )
	If hour_twelve = 0 Then hour_twelve = 12	
	Dim date_parts : date_parts = Array( DatePart("yyyy",oDate) , Right(DatePart("yyyy",oDate) , 2) , Right("0" & DatePart("m",oDate),2) , DatePart("m",oDate) , Right("0" & DatePart("d",oDate),2) , DatePart("d",oDate) , Right("0" & DatePart("h",oDate),2) , DatePart("h",oDate) ,  Right("0" & hour_twelve,2) , hour_twelve , Right("0" & DatePart("n",oDate),2) , DatePart("n",oDate) , Right("0" & DatePart("s",oDate),2) , DatePart("s",oDate) , DatePart("q",oDate)  , Right("0" & DatePart("y",oDate),3) , DatePart("y",oDate) , DatePart("w",oDate) ,  Right("0" & DatePart("ww",oDate),2) , DatePart("ww",oDate) , WeekdayName(DatePart("w",oDate),False) , WeekdayName(DatePart("w",oDate),True) , MonthName(Datepart("m",oDate),False) , MonthName(Datepart("m",oDate),True) , meridiem(Abs(DatePart("h",oDate) >= 12)) , LCase(meridiem(Abs(DatePart("h",oDate) >= 12))) )
	Dim i
	For i = 0 To UBound(parse_patterns)
		strftime = Replace( strftime , parse_patterns(i) , date_parts(i),1,Len(strftime),vbBinaryCompare )
	Next
End Function

Function TrimL(s)
	Dim r : Set r = New RegExp
	r.Global=False
	r.IgnoreCase=True
	r.Pattern="^\s*"
	TrimL=r.Replace(s,"")
End Function

Function TrimR(s)
	Dim r : Set r = New RegExp
	r.Global=False
	r.IgnoreCase=True
	r.Pattern="\s*$"
	TrimR=r.Replace(s,"")
End Function

Function TrimB( s )
	TrimB=TrimR(TrimL(s))
End Function

' Function returns true on success, otherwise false
Function ini_write( sAppName, sKeyName, sString, sFileName )    	
	ini_write=False
	If IsNull(sAppName) Then Exit Function
	On Error Resume Next
	' Zero = Searching , 1=Found , 2=Past
	Dim inSection : inSection = 0
	Dim atKey : atKey = 0
	Dim tokenType : tokenType = 0
	Dim lastKey : lastKey = 0
	Err.Clear
	Dim oFile : Set oFile = objFSO.OpenTextFile( sFileName , ForReading , False , TriStateAscii )        
	If Err.Number = 0 Then
		Dim regexsection : Set regexsection = New RegExp 
		Dim regexkey : Set regexkey = New RegExp
		Dim regexcomment : Set regexcomment = New RegExp
		Dim i, lines : lines = Array
		regexcomment.Pattern = "^([\s]*[#|;].*)"
		regexcomment.Global = False
		regexcomment.IgnoreCase = True
		regexcomment.MultiLine = False
		regexsection.Pattern = "^[\s]*\[[\s]*([^\[\s].*[^\s\]])[\s]*\][\s]*$"        						
		regexsection.Global = False
		regexsection.IgnoreCase = True
		regexsection.MultiLine = False
		regexkey.Pattern = "^\s*([^=]*)\s*=(.*)$"
		regexkey.Global = False
		regexkey.IgnoreCase = True
		regexkey.MultiLine = False
		Do While(Not oFile.AtEndOfStream)
			Do While(Not oFile.AtEndOfStream)
				Dim line : line = oFile.ReadLine()
				Dim line_trimmed : line_trimmed = TrimB(line)
				i = UBound(lines) + 1 ' Track current index
				If line_trimmed <> vbNullString Then
					Dim m
					If regexsection.Test(line) Then
						Set m = regexsection.Execute(line)
						If inSection=1 Then inSection = 2 ' No longer in the section of interest
						If inSection=0 Then
							If StrComp( m.Item(0).subMatches.Item(0) , TrimB(sAppName) , vbTextCompare ) = 0 Then
								inSection = 1 ' Found Section
								lastKey=i ' Set next key position
								' We remove the section if the keyname is null
								If IsNull(sKeyName) Then
									Exit Do ' Section Delete + All Keys
								End If
							End If
						End If  
					ElseIf Not regexcomment.Test(line) And Not IsNull(sKeyName) Then
						Dim k
						If regexkey.Test(line) Then
							Set m = regexkey.Execute(line)
							k = TrimB(m.Item(0).Submatches.Item(0))
						Else 
							k = TrimB(line)
						End If
						If inSection = 1 Then
							lastKey=i ' Track last key in section
							' Did we find the key? if not check
							If atKey=0 And StrComp( k , TrimB(sKeyName) , vbTextCompare ) = 0 Then 
								atKey = 1 ' Found Key
								If IsNull(sString) Then
									Exit Do ' Delete key if value is null
								Else
									line=k&"="&sString ' Otherwise set the key
								End If
							End If
						End If
					End If
				End If				
				If inSection=1 And IsNull(sKeyName) Then Exit Do 'Delete all values up to next section
				ReDim Preserve lines(i)
				lines(i)=line
			Loop	
		Loop
		oFile.Close()
	End If       
	'WScript.Echo "InSection: "& inSection
	'WScript.Echo "Pos LastKey: " & lastKey
	'WScript.Echo "Key Found: " & atKey	
	'WScript.Quit 0
	'If ( inSection > 0 And atKey=1 And Not IsNull(sKeyName) And Not IsNull(sString) ) Then WScript.Echo "Update File Key" 
	'If ( inSection > 0 And (atKey=0 Or atKey=1) And Not IsNull(sKeyName) And Not IsNull(sString) ) Then WScript.Echo "Update File Key" 
	'If ( inSection = 0 And Not IsNull(sKeyName) And Not IsNull(sString) ) Then WScript.Echo "Update File Section" 
	'If ( inSection > 0 And (IsNull(sKeyName) Or IsNull(sString)) ) Then WScript.Echo "Shit Removed"
	If ( Not objFSO.FileExists( sFileName ) ) Then WScript.Echo "File Not Exists"
		If ( Not objFSO.FileExists( sFileName ) ) Or ( inSection > 0 And (IsNull(sKeyName) Or IsNull(sString)) ) Or ( inSection = 0 And Not IsNull(sKeyName) And Not IsNull(sString) ) Or ( inSection > 0 And (atKey=0 Or atKey=1) And Not IsNull(sKeyName) And Not IsNull(sString) ) Then
			Set oFile = objFSO.OpenTextFile( sFileName , ForWriting , True , TriStateAscii )
			If Err.Number = 0 Then   			
				For i=0 To UBound(lines) 
					oFile.WriteLine(lines(i))
	 				'Add key to section if it exits
					If inSection > 0 And atKey=0 And Not IsNull(sKeyName) And Not IsNull(sString) And i=lastKey Then Call oFile.WriteLine(sKeyName&"="&sString)
				Next
	 			'Create section and key
				If inSection = 0 And Not IsNull(sKeyName) And Not IsNull(sString) Then 
					Call oFile.WriteLine("["&sAppName&"]")
					Call oFile.WriteLine(sKeyName&"="&sString)
				End If
				ini_write=True
				oFile.Close()
			End If
		End If
End Function

Function ini_read( sAppName, sKeyName, sDefault , sString, sFileName )
		ini_read=False
		On Error Resume Next
        Dim bFoundSection : bFoundSection = False
	    Dim bKeyFound : bKeyFound = False
        Err.Clear
        Dim oReader : Set oReader = objFSO.OpenTextFile( sFileName , ForReading , False , TriStateAscii ) 
        If Err.Number = 0 Then
	        Dim regexsection : set regexsection = new regexp 
	        Dim regexkey : Set regexkey = new regexp
			Dim regexcomment : Set regexcomment = new regexp
			If IsNull(sAppName) Or IsNull(sKeyName) Then sString=Array
			Dim n
	        regexcomment.Pattern = "^([\s]*[#|;].*)"
			regexcomment.Global = False
			regexcomment.IgnoreCase = True
			regexcomment.MultiLine = False
	        regexsection.Pattern = "^[\s]*\[[\s]*([^\[\s].*[^\s\]])[\s]*\][\s]*$"        						
			regexsection.Global = False
			regexsection.IgnoreCase = True
			regexsection.MultiLine = False
			regexkey.Pattern = "^\s*([^=]*)\s*=(.*)$"
			regexkey.Global = False
			regexkey.IgnoreCase = True
			regexkey.MultiLine = False	
	        Do While(Not oReader.AtEndOfStream)
	        	Dim line : line = oReader.ReadLine()
	        	Dim line_trimmed : line_trimmed = TrimB(line)
				If line_trimmed <> vbNullString Then
	            	Dim m
					If regexsection.Test(line) Then
	                	Set m = regexsection.Execute(line)
	                	If IsNull(sAppName) Then
	                		ini_read=True
	        				n = UBound(sString) + 1		
							ReDim Preserve sString(n)
							sString(n)=m.Item(0).subMatches.Item(0)
						Else
							If bFoundSection Then Exit Do
	                    	If StrComp( m.Item(0).subMatches.Item(0) , TrimB(sAppName) , vbTextCompare ) = 0 Then bFoundSection=True
	                	End If 
	                ElseIf Not regexcomment.Test(line) And bFoundSection Then ' If it's not a comment it has to be a key
	                	Dim k : k = vbNullString
	                	Dim v : v = vbNullString
						If regexkey.Test(line) Then 
							set m = regexkey.Execute(line)
							k = TrimB(m.Item(0).Submatches.Item(0))
							v = m.Item(0).Submatches.Item(1)
						Else 
							k = TrimB(line)
						End If
						If IsNull(sKeyName) Then
	                		ini_read=True
	                		n = UBound(sString) + 1		
							ReDim Preserve sString(n)
							sString(n)=TrimB(k)
						Else
	                    	If StrComp( k , TrimB(sKeyName) , vbTextCompare ) = 0 Then
	                    		bKeyFound=True
	                    		ini_read=True
	                    		sString=v
	                    		Exit Do
	                    	End If 
						End If
	                End If                            
	            End If
	        Loop
	        oReader.Close()
	    End If
        If Not IsNull(sKeyName) And Not IsNull(sAppName) And Not bKeyFound Then 
        	ini_read=True
        	sString=sDefault
        End If
End Function

' Provides cached logon data for group membership validation
Class std_logoninfo
	Private Sub Class_Initialize()
		Set m_Groups = CreateObject("Scripting.Dictionary")
		m_Groups.CompareMode = vbTextCompare
		m_sHomeDrive = vbNullString
		m_sHomeDir = vbNullString
	End Sub
	' Function prefetches the group data for the current logged on user
	' the data is used to populate an internal dictionary. This function
	' returns true if the group data was loaded
	Public Function load_data()
		On Error Resume Next
		load_data = False
		Dim objUser : Set objUser = Nothing
	 	Dim objADSysInfo : Set objADSysInfo = CreateObject("ADSystemInfo")
		' Primary Group Query
		Err.Clear
        Set objUser = GetObject(objShell.ExpandEnvironmentStrings("WinNT://%USERDOMAIN%/%USERNAME%,user"))
			If Not objUser Is Nothing And Err.Number = 0 Then
             For Each group In objUser.Groups
                    Call m_Groups.Add( group.name , "" )  
            Next
        End If
        Set objUser = Nothing
		' Query all the other groups
	 	Err.Clear
		Set objUser = GetObject("GC://" & objADSysInfo.UserName)
		If Not objUser Is Nothing And Err.Number = 0 Then
			WScript.StdOut.Write("Loading.")
			load_data=True
			Dim group
			m_sHomeDir = objUser.homeDirectory
			m_sHomeDrive = objUser.homeDrive
			For Each group In objUser.Groups
				' Pre W2K Name
				Call m_Groups.Add( group.sAMAccountName , "" )
				' Post W2K Name
				Call m_Groups.Add( group.cn , "" )  
			Next
		End If
		Err.Clear
	End Function
	Public Property Get HomeDir
		homedir = m_sHomeDir
	End Property
	Public Property Get HomeDrive
		homedrive = m_sHomeDrive
	End Property 
	' Function returns true if the group is stored in the cache
	Public Function IsMember( strGroup )
		IsMember = CBool(m_Groups.Exists( Trim(strGroup)))
	End Function
	Public Property Get Groups
		Groups = m_Groups.Keys
	End Property
	' Members
	Private m_Groups
	Private m_sHomeDir
	Private m_sHomeDrive
End Class

Class std_hostfile
	Private Sub Class_Initialize()
		Set m_lines = CreateObject("Scripting.Dictionary")
		Set m_ip_map = CreateObject("Scripting.Dictionary")
		Set m_alias_map = CreateObject("Scripting.Dictionary")
		m_alias_map.CompareMode = vbTextCompare
	End Sub
	Private sub Class_Terminate()
		Set m_lines = Nothing
		Set m_ip_map = Nothing
		Set m_alias_map = Nothing
	End Sub
	' Function returns the ip and data
	Private Sub parse_line( ByVal line , ByRef comment , ByRef ip , ByRef aliases  ) 
		Dim rx : Set rx = New RegExp
		Dim r
		rx.Global = False
		rx.IgnoreCase = True
		rx.Pattern = "\s*(#.*)\s*"
		' Parse Comment
		If rx.Test( line ) Then
			Set r = rx.Execute( line )
			comment = r.Item(0).subMatches.Item(0)
			line = rx.Replace( line , "" )
		End If		
		rx.Pattern = "\s*((\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b)|(\s*((([0-9A-F]{1,4}:){7}([0-9A-F]{1,4}|:))|(([0-9A-F]{1,4}:){6}(:[0-9A-F]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-F]{1,4}:){5}(((:[0-9A-F]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-F]{1,4}:){4}(((:[0-9A-F]{1,4}){1,3})|((:[0-9A-F]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){3}(((:[0-9A-F]{1,4}){1,4})|((:[0-9A-F]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){2}(((:[0-9A-F]{1,4}){1,5})|((:[0-9A-F]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){1}(((:[0-9A-F]{1,4}){1,6})|((:[0-9A-F]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-F]{1,4}){1,7})|((:[0-9A-F]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*))\s*"
		' Parse IP
		If rx.Test( line ) Then
			Set r = rx.Execute( line )
			ip = r.Item(0).subMatches.Item(0)
			aliases = rx.Replace( line , "" )
		End If
	End Sub
	' Function returns the ip and data
	Private Function getip( ByRef str , ByRef ip  ) 
		getip = False 
		Dim rx_ip : Set rx_ip = New RegExp
		rx_ip.Global = False
		rx_ip.IgnoreCase = True
		rx_ip.Pattern = "\s*((\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b)|(\s*((([0-9A-F]{1,4}:){7}([0-9A-F]{1,4}|:))|(([0-9A-F]{1,4}:){6}(:[0-9A-F]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-F]{1,4}:){5}(((:[0-9A-F]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-F]{1,4}:){4}(((:[0-9A-F]{1,4}){1,3})|((:[0-9A-F]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){3}(((:[0-9A-F]{1,4}){1,4})|((:[0-9A-F]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){2}(((:[0-9A-F]{1,4}){1,5})|((:[0-9A-F]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){1}(((:[0-9A-F]{1,4}){1,6})|((:[0-9A-F]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-F]{1,4}){1,7})|((:[0-9A-F]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*))\s*"
		If rx_ip.Test( str ) Then
			Dim r : Set r = rx_ip.Execute( str )
			ip = r.Item(0).subMatches.Item(0)
			str = rx_ip.Replace( str , "" )
			getip = True
		End If
	End Function
	' Internal function used to validate IPV4 addresses
	Private Function isipv4( ByVal ip )
		Dim rx_ipv4 : Set rx_ipv4 = New RegExp
		rx_ipv4.Global = false
		rx_ipv4.IgnoreCase = True
		rx_ipv4.Pattern = "^\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b$"
		isipv4 = rx_ipv4.Test( ip )
	End Function
	' Internal function used to validate IPV6 addresses
	Private Function isipv6( ip )
		Dim rx_ipv6 : Set rx_ipv6 = New RegExp
		rx_ipv6.Global = false
		rx_ipv6.IgnoreCase = True
		rx_ipv6.Pattern = "^\s*((([0-9A-F]{1,4}:){7}([0-9A-F]{1,4}|:))|(([0-9A-F]{1,4}:){6}(:[0-9A-F]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-F]{1,4}:){5}(((:[0-9A-F]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-F]{1,4}:){4}(((:[0-9A-F]{1,4}){1,3})|((:[0-9A-F]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){3}(((:[0-9A-F]{1,4}){1,4})|((:[0-9A-F]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){2}(((:[0-9A-F]{1,4}){1,5})|((:[0-9A-F]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-F]{1,4}:){1}(((:[0-9A-F]{1,4}){1,6})|((:[0-9A-F]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-F]{1,4}){1,7})|((:[0-9A-F]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$"			
		IsIPV6 = rx_ipv6.Test( ip )
	End Function
	' Internal function used to generate unique IDs
	Private Function genguid()
		Dim guidgen : Set guidgen = CreateObject("Scriptlet.TypeLib") 
		genguid = Mid(guidgen.Guid, 2, 36)
	End Function
	' Function returns true on success, otherwise false if the ip doesn't exist
	Public Function GetHostEntryAliases( ByVal ip , ByRef aliases )
		GetHostEntryAliases = False
		aliases = Array
		If m_ip_map.Exists( ip ) Then 
			aliases = m_lines.Item( m_ip_map.Item(ip) )(1).Keys
			GetHostEntryAliases = True
		End If
	End Function 
	' Function returns true on success, otherwise false if alias doesn't exist
	Public Function GetHostEntryAliasAddresses( ByVal alias , ByRef ips )
		GetHostEntryAliasAddresses = False
		ips = Array
		If m_alias_map.Exists( alias ) Then 
			ips = m_alias_map.Item( alias ).Keys
			GetHostEntryAliasAddresses = True
		End If
	End Function 
	' Removes a host entry by IPV4 or IPV6 address, return true
	' on success, otherwise false if the IP is doesn't exist
	Public Function DeleteHostEntry( ByVal ip )
		DeleteHostEntry = False
		If m_ip_map.Exists( ip ) Then 
			Dim a
			Dim uid : uid = m_ip_map.Item(ip)
			' Remove the ip's from the associated aliases
			For Each a In m_lines.Item(uid)(1).Keys 
				If m_alias_map.Item( a ).Exists( ip ) Then
					m_alias_map.Item( a ).Remove( ip )
				End If
				' If there are no more assoicated IPs remove the alias
				If m_alias_map.Item(a).Count = 0 Then
					m_alias_map.Remove( a )
				End If 
			Next
			' This *should* exist since we manage the entries and mappings
			m_lines.Remove( uid )
			m_ip_map.Remove( ip )
			DeleteHostEntry = True
		End If
	End Function
	' Removes a host alias by IPV4 or IPV6 address
	Public Function DeleteHostEntryAlias( ByVal ip , ByVal alias )
		DeleteHostEntryAlias = False
		' If the IP is valid
		If m_ip_map.Exists( ip ) Then
			Dim uid : uid = m_ip_map.Item(ip)		
			' If the alias exists remove the ip and if no more ip's are mapped to the alias remove the alias	
			If m_alias_map.Exists( alias ) Then
				If m_alias_map.Item( alias ).Exists( ip ) Then
					m_alias_map.Item( alias ).Remove( ip )
				End If
				If m_alias_map.Item(alias).Count = 0 Then
					m_alias_map.Remove( alias )
				End If
			End If
			' If IP no longer has aliases associated with it remove it
			Call delalias( uid , alias )
			If m_lines.Item( uid )(1).Count = 0 Then
				m_lines.Remove( uid )
				m_ip_map.Remove( ip )
			End If
			DeleteHostEntryAlias = True
		End If
	End Function
	' Adds a host entry by IPV4 or IPV6 address, alias should be 
	' the text alias for the address. Returns true on success
	Public Function AddHostEntry( ByVal ip , ByVal alias )		
			AddHostEntry = False 		
			Dim rx : Set rx = New RegExp
			rx.Global = true
			rx.IgnoreCase = True
			rx.Pattern = "\s*"
			alias = rx.Replace( alias , "" )
			ip = rx.Replace( ip , "" )
			' Validate IP
			If isipv6( ip ) Or isipv4( ip ) Then 
				' Check for alias in the alias mapping
				If Not m_alias_map.Exists( alias ) Then	Call m_alias_map.Add( alias , CreateObject("Scripting.Dictionary") )
				If Not m_alias_map.Item( alias ).Exists( ip ) Then Call m_alias_map.Item( alias ).Add( ip , "" )
				' Map IP -> alias 
				If m_ip_map.Exists( ip ) Then 
					' Lookup the index by ip then add aliases
					Call addalias( m_ip_map.Item(ip) , alias )
				Else
					' Store File Line
					Dim uid : uid = genguid
					Call m_lines.Add( uid , Array( ip , CreateObject("Scripting.Dictionary") , vbNullString ) )
					Call addalias( uid , alias )
					Call m_ip_map.Add( ip , uid )
				End If
				AddHostEntry = True
			End If	
	End Function
	' Should be used for debugging the data
	Public Sub DumpData( )
		Dim id, a, ip, a_map
		' Debug dump host file out to text
		WScript.Echo( "-------------------------------------------------------------------" )
		For Each id In m_lines.Keys
			If TypeName( m_lines.Item(id) )  = "String" Then
				WScript.Echo m_lines.Item(id)
			ElseIf TypeName( m_lines.Item(id) ) = "Variant()" Then
				WScript.StdOut.Write lpad( m_lines.Item(id)(0) & "" , " " , 16 )  & Space(8)
				For Each a In m_lines.Item(id)(1).Keys
					WScript.StdOut.Write a & " " 
				Next
					WScript.StdOut.Write m_lines.Item(id)(2) & vbCrLf
			End If 
		Next
		WScript.Echo( "-------------------------------------------------------------------" )
		
		For Each ip In m_ip_map.Keys
			For Each a In m_lines.Item( m_ip_map.Item(ip) )(1).Keys
				WScript.Echo "IP [" & ip & "] ID Map {" & m_ip_map.Item(ip)	& "} --> alias [" & a & "]"
				If m_alias_map.Exists( a ) Then 
					For Each a_map In m_alias_map.Item( a ).Keys
						WScript.Echo "alias Key [" & a & "] IP --> [" & a_map & "]"
					Next
				End If
			Next
		Next	 
	End Sub 
	' Internal formattig function for padding host data
	Private Function lpad ( str , padch , padlen ) 
		If padlen - Len(str) >= 0 Then 
			Lpad = String(padlen - Len(str),padch) & str 
		Else
			Lpad = str	
		End If 	
	End Function
	' Returns all the IP addresses defined in the host file
	' Returns true on success, otherwise false
	Public Function GetAllHostEntryAddresses( ByRef ips )
		GetAllHostEntryAddresses = False
		ips = Array
		If m_ip_map.Count > 0 Then 
			ips = m_ip_map.Keys
			GetAllHostEntryAddresses = True
		End If
	End Function
	' Returns all the aliases defined in the host file
	' Returns true on success, otherwise false
	Public Function GetAllHostEntryAliases( ByRef aliases )
		GetAllHostEntryAliases = False
		aliases = Array
		If m_alias_map.Count > 0 Then 
			aliases = m_alias_map.Keys
			GetAllHostEntryAliases = True
		End If
	End Function
	
	' Write host file returns true if file could be opened for writing
	Public Function Save( ByVal hostfile )		
		On Error Resume Next
		Save = False
		Dim id, a
		Dim objFSO : Set objFSO = CreateObject("Scripting.FileSystemObject")
		Dim oFile : Set oFile = Nothing
		Err.Clear
		Set oFile = objFSO.OpenTextFile( hostfile , ForWriting , True , TriStateAscii )
		If Err.Number = 0 Then 
			For Each id In m_lines.Keys
				If TypeName( m_lines.Item(id) ) = "String" Then
					Call oFile.WriteLine( m_lines.Item(id) )
				ElseIf TypeName( m_lines.Item(id) ) = "Variant()" Then
					Call oFile.Write( lpad( m_lines.Item(id)(0) & "" , " " , 16 )  & Space(8) )
					For Each a In m_lines.Item(id)(1).Keys
						Call oFile.Write( a & " " )
					Next
					Call oFile.Write( m_lines.Item(id)(2) & vbCrLf )
				End If 
			Next
			Save = True
		End If
		Set objFSO = Nothing
	End Function
	
	' Write host file returns true if file could be opened for reading
	Public Function Load( ByVal hostfile , ByVal bmergecomments )
		On Error Resume Next		
		Load = False
		m_lines.RemoveAll()
		m_ip_map.RemoveAll()
		m_alias_map.RemoveAll()
		Dim rx : Set rx = New RegExp
		rx.Global = true
		rx.IgnoreCase = True
		rx.Pattern = "\s+"
		Dim objFSO : Set objFSO = CreateObject("Scripting.FileSystemObject")
		Dim oFile : Set oFile=Nothing
		Err.Clear
		Set oFile = objFSO.OpenTextFile( hostfile , ForReading , False , TriStateAscii )
		If Err.Number = 0 Then 
		While Not oFile.AtEndOfStream
			Dim pos : pos = 0
			' Data will not be modified to preserve the file context
			Dim data : data = Trim(oFile.ReadLine())
			' Remove all the extra whitespace so we have a single spacing	
			' Line will be chopped up to see valid information exists
			Dim line : line = Trim(rx.Replace(data, " "))
			Dim ip : ip = vbNullString
			Dim comment : comment = vbNullString
			Dim aliases : aliases = vbNullString
			' Check if the line is empty
			If line <> vbNullString Then					
				Call parse_line( line , comment , ip , aliases )
				If isipv4( ip ) Or isipv6( ip ) Then 	
					Dim a, uid
					' Map alias -> IPs
					For Each a In Split(aliases)
						If Not m_alias_map.Exists( a ) Then 
							Call m_alias_map.Add( a , CreateObject("Scripting.Dictionary") )
						End If
						If Not m_alias_map.Item( a ).Exists( ip ) Then Call m_alias_map.Item( a ).Add( ip , "" )
					Next
					' Map IP -> alias 
					If m_ip_map.Exists( ip ) Then
						uid = m_ip_map.Item(ip)
						' Lookup the index by ip then add aliases
						For Each a In Split(aliases)
							Call addalias( uid , a )
						Next
						If bmergecomments Then 
							' Overkill (should be first # )
							pos = InStr( 1, comment , "#" , vbTextCompare )
							If pos <> 0 Then 
								' Replace leading # from dual comment
								comment = Mid( comment , pos + 1 )
								' Merge comments
								Call setcomment( uid , m_lines.Item( uid )(2) & "," & comment )
							End If
						End If
					Else
						' Store File Line
						uid = genguid
						Call m_lines.Add( uid , Array( ip , CreateObject("Scripting.Dictionary") , comment ) )
						For Each a In Split(aliases)
							Call addalias( uid , a )
						Next
						Call m_ip_map.Add( ip , uid )
					End If
				Else ' Unknown IP format or malformed file
					Call m_lines.Add(genguid,data)
				End If
			Else
				Call m_lines.Add(genguid,data)
			End If
		Wend
		Load=True
		End If
		Set objFSO = Nothing
	End Function
	Private Function setip( uid , ip )
		setip = False
		If m_lines.Exists( uid ) Then
			m_lines.Item(uid) = Array( ip , m_lines.Item(uid)(1) , m_lines.Item(uid)(2) ) 
			setip = True
		End If
	End Function
	Private Function setcomment( uid , comment )
		setcomment = False
		If m_lines.Exists( uid ) Then
			Dim a : a = m_lines.Item(uid)
			m_lines.Item(uid) = Array( m_lines.Item(uid)(0) , m_lines.Item(uid)(1) , comment )  
			setcomment = True
		End If
	End Function
	Private Function addalias( uid , alias )
		addalias = False
		If m_lines.Exists( uid ) Then
			If Not m_lines.Item(uid)(1).Exists( alias ) Then 
				Call m_lines.Item(uid)(1).Add( alias , "" )
				addalias = True
			End If
		End If
	End Function
	Private Function delalias( uid , alias )
		delalias = False
		If m_lines.Exists( uid ) Then
			If m_lines.Item(uid)(1).Exists( alias ) Then 
				Call m_lines.Item(uid)(1).Remove( alias )
				delalias = CBool( Not m_lines.Item(uid)(1).Exists( alias ) )
			End If
		End If
	End Function
	Private m_lines
	Private m_ip_map
	Private m_alias_map
End Class

' Service helper class allows creation, deletion, and manipulation of local or remote services.		
Class std_service
	Private Sub Class_Initialize()
		Set objWMIService = Nothing
	End Sub
	
	' Connect to the reg provider for this registy object
	Public Function ConnectProvider( sComputerName )
		ConnectProvider = False
		On Error Resume Next
		Err.Clear
		Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & sComputerName & "\root\cimv2")
		If Err.Number = 0 Then			
			ConnectProvider = True
		End If
	End Function
	
	' Issue a service stop where strService is the service name
	' Return error codes can be SERVICE_WMI_FAIL, SERVICE_NOT_EXIST
	' In all cases success returns SERVICE_SUCCESS
	Public Function ServiceStop( strService , bQueryByDisplayName )
		ServiceStop = SERVICE_WMI_FAIL
		Dim objService : Set objService = Nothing
		If Not objWMIService Is Nothing Then
			If bQueryByDisplayName Then
				On Error Resume Next
				Dim colServices : Set colServices = objWMIService.ExecQuery("Select * from Win32_Service WHERE DisplayName='"& strService &"'")
				For Each objService In colServices
					Exit For
				Next
			Else
				On Error Resume Next
				Set objService = objWMIService.Get("Win32_Service.Name='" & strService & "'")
			End If 
			If objService Is Nothing Then 
				ServiceStop = SERVICE_NOT_EXIST
			Else
				If objService.State <> SERVICE_STAT_STOPPED Then
					Dim rVal
					rVal = objService.StopService()
					If rVal = SERVICE_SUCCESS Or rVal = SERVICE_STATE_REQUESTED Then
						ServiceStop = SERVICE_SUCCESS
					Else
						ServiceStop = rVal
					End If
				Else
					ServiceStop = SERVICE_SUCCESS
				End If 
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set objService = Nothing
	End Function
	
	' Issue a service start where strService is the service name
	' Return error codes can be SERVICE_WMI_FAIL, SERVICE_NOT_EXIST
	' In all cases success returns SERVICE_SUCCESS
	Public Function ServiceStart( strService , bQueryByDisplayName )
		ServiceStart = SERVICE_WMI_FAIL
		Dim objService : Set objService = Nothing
		If Not objWMIService Is Nothing Then
			If bQueryByDisplayName Then 
				On Error Resume Next
				Dim colServices : Set colServices = objWMIService.ExecQuery("Select * from Win32_Service WHERE DisplayName='"& strService &"'")
				For Each objService In colServices
					Exit For
				Next
			Else
				On Error Resume Next
				Set objService = objWMIService.Get("Win32_Service.Name='" & strService & "'")
			End If 
			If objService Is Nothing Then 
				ServiceStart = SERVICE_NOT_EXIST
			Else
				If objService.State <> SERVICE_STAT_RUNNING Then
					Dim rVal
					rVal = objService.StartService()
					If rVal = SERVICE_SUCCESS Or rVal = SERVICE_STATE_REQUESTED Then
						ServiceStart = SERVICE_SUCCESS
					Else
						ServiceStart = rVal
					End If
				Else
					ServiceStart = SERVICE_SUCCESS
				End If 
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set objService = Nothing
	End Function
	
	
	' Query the service status where strService is the service name
	' and strStatus is the text returned status string
	' Return error codes can be SERVICE_WMI_FAIL, SERVICE_NOT_EXIST 
	' In all cases success returns SERVICE_SUCCESS
	Public Function ServiceStatus( strService , byRef strStatus , bQueryByDisplayName )
		ServiceStatus = SERVICE_WMI_FAIL
		strStatus = "SERVICE_WMI_FAIL"
		Dim objService : Set objService = Nothing
		If Not objWMIService Is Nothing Then
			If bQueryByDisplayName Then
				On Error Resume Next
				Dim colServices : Set colServices = objWMIService.ExecQuery("Select * from Win32_Service WHERE DisplayName='"& strService &"'")
				For Each objService In colServices
					Exit For
				Next
			Else
				On Error Resume Next
				Set objService = objWMIService.Get("Win32_Service.Name='" & strService & "'")
			End If
			If objService Is Nothing Then 
				ServiceStatus = SERVICE_NOT_EXIST
				strStatus = "SERVICE_NOT_EXIST"
			Else
				strStatus = objService.State
				ServiceStatus = SERVICE_SUCCESS
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set objService = Nothing
	End Function
	
	' Wait for a service state where strService is the service name
	' strState should be one the constants below:
	' SERVICE_STAT_STOPPED
	' SERVICE_STAT_RUNNING
	' SERVICE_STAT_PAUSED
	' nTimeoutSec should be the number of seconds to wait till deeming the service
	' state stuck or not changeable.
	' The the function returns false if the service state we reached and false
	' if the service state was not reached.
	Public Function ServiceStatusWait( strService , bQueryByDisplayName , strState , nTimeoutSec )
		ServiceStatusWait = False
		Dim Timeout : Timeout = 0
		Dim state , rval
		If nTimeoutSec < 1 Then 
			rval = ServiceStatus( strService , state , bQueryByDisplayName )
			If rval = SERVICE_SUCCESS And UCase(state) = UCase(strState) Then
				ServiceStatusWait = True
			Else
				ServiceStatusWait = False
			End If
		Else
			For Timeout = 0 To nTimeoutSec - 1
				rval = ServiceStatus( strService , state , bQueryByDisplayName )
				If rval = SERVICE_SUCCESS And UCase(state) = UCase(strState) Then 
					ServiceStatusWait = True
					Exit For
				End If 
				If rval = SERVICE_NOT_EXIST Then
					ServiceStatusWait = False
					Exit For
				End If
				WScript.Sleep(1000)
			Next
		End If 
	End Function
	
	' Issue a service deletion where strService is the service name
	' Return error codes can be SERVICE_WMI_FAIL, SERVICE_NOT_EXIST 
	' In all cases success returns SERVICE_SUCCESS
	Public Function ServiceDelete( strService , bQueryByDisplayName )
		ServiceDelete = SERVICE_WMI_FAIL
		Dim objService : Set objService = Nothing
		If Not objWMIService Is Nothing Then
			If bQueryByDisplayName Then
				On Error Resume Next
				Dim colServices : Set colServices = objWMIService.ExecQuery("Select * from Win32_Service WHERE DisplayName='"& strService &"'")
				For Each objService In colServices
					Exit For
				Next
			Else
				On Error Resume Next
				Set objService = objWMIService.Get("Win32_Service.Name='" & strService & "'")
			End If
			If objService Is Nothing Then 
				ServiceDelete = SERVICE_NOT_EXIST
			Else
				ServiceDelete = objService.Delete()
			End If
		End If 
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set objService = Nothing
	End Function
	
	' Create a service where strServiceName is the service name
	' See the section "TEMPLATE CONSTANTS" at the top of the script for "Service Types" and "Error Control" for
	' values to be used with nServiceType and nErrorControl
	' Return error codes can be SERVICE_WMI_FAIL, SERVICE_NOT_EXIST 
	' In all cases success returns SERVICE_SUCCESS
	Public Function ServiceCreate( strServiceName , strDescription , strSvcExe , nServiceType , nErrorControl , strStartMode , bDesktopInteract , strStartName , strStartPass , strLoadOrderGroup )
		ServiceCreate = SERVICE_WMI_FAIL
		Dim objService
		If Not objWMIService Is Nothing Then
			On Error Resume Next
			Set objService = objWMIService.Get("Win32_BaseService")
			If objService Is Nothing Then 
				ServiceCreate = SERVICE_NOT_EXIST ' Not sure why whis would happen
			Else
				ServiceCreate = objService.Create( strServiceName , strDescription , strSvcExe , nServiceType , nErrorControl , strStartMode , bDesktopInteract, strStartName , strStartPass )
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set objService = Nothing
	End Function
	
	
	' Query service start mode where strService is the service name
	' strMode is returned byRef if the function completes with success 
	' Return error codes can be SERVICE_WMI_FAIL, SERVICE_NOT_EXIST
	' In all cases success returns SERVICE_SUCCESS
	' See the "TEMPLATE CONSTANTS" section for a list of Service Start Modes:
	' SERVICE_START_BOOT
	' SERVICE_START_SYSTEM
	' SERVICE_START_AUTO
	' SERVICE_START_MANUAL
	' SERVICE_START_DISABLE
	Public Function ServiceStartMode( strService, byRef strMode , bQueryByDisplayName )
		ServiceStartMode = SERVICE_WMI_FAIL
		strMode = "SERVICE_WMI_FAIL"
		Dim objService : objService = Nothing
		If Not objWMIService Is Nothing Then
			If bQueryByDisplayName Then
				On Error Resume Next
				Dim colServices : Set colServices = objWMIService.ExecQuery("Select * from Win32_Service WHERE DisplayName='"& strService &"'")
				For Each objService In colServices
					Exit For
				Next
			Else
				On Error Resume Next
				Set objService = objWMIService.Get("Win32_Service.Name='" & strService & "'")
			End If
			If objService Is Nothing Then 
				ServiceStartMode = SERVICE_NOT_EXIST
				strMode = "SERVICE_NOT_EXIST"
			Else
				strMode = objService.StartMode
				ServiceStartMode = SERVICE_SUCCESS
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set objService = Nothing
	End Function
	
	
	' Modify service start mode where strService is the service name
	' strMode is the start mode which will be applied to the service if it exists
	' Return error codes can be SERVICE_WMI_FAIL, SERVICE_NOT_EXIST
	' In all cases success returns SERVICE_SUCCESS
	' See the "TEMPLATE CONSTANTS" section for a list of Service Start Modes:
	' SERVICE_START_BOOT
	' SERVICE_START_SYSTEM
	' SERVICE_START_AUTO
	' SERVICE_START_MANUAL
	' SERVICE_START_DISABLE
	Public Function ChangeStartMode( strService , strMode , bQueryByDisplayName )
		ChangeStartMode = SERVICE_WMI_FAIL
		Dim objService : objService = Nothing
		If Not objWMIService Is Nothing Then
			If bQueryByDisplayName Then
				On Error Resume Next
				Dim colServices : Set colServices = objWMIService.ExecQuery("Select * from Win32_Service WHERE DisplayName='"& strService &"'")
				For Each objService In colServices
					Exit For
				Next
			Else
				On Error Resume Next
				Set objService = objWMIService.Get("Win32_Service.Name='" & strService & "'")
			End If
			If objService Is Nothing Then 
				ChangeStartMode = SERVICE_NOT_EXIST
			Else
				ChangeStartMode = objService.ChangeStartMode(strMode)
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set objService = Nothing
	End Function
	
	' Members
	Private objWMIService
End Class

' Basic process class to help managed local or remote processes.
' This class should be expanded to do other functions in the future
Class std_process
	' Must be nothing on creation
	Private Sub Class_Initialize()
		Set objWMIService = Nothing
	End Sub
	
	' Connect to the reg provider for this registy object
	Public Function ConnectProvider( sComputerName )
		ConnectProvider = False
		On Error Resume Next
		Err.Clear
		Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & sComputerName & "\root\cimv2")
		If Err.Number = 0 Then			
			ConnectProvider = True
		End If
	End Function
	
	' Returns tr if a processes of strProc exist.
	' ie: strProc is the process name like calc.exe
	Function ProcessExists( strProc )
		ProcessExists = PROCESS_WMI_FAIL
		Dim colProcessList	
		If Not objWMIService Is Nothing Then
			Set colProcessList = objWMIService.ExecQuery("Select * from Win32_Process WHERE Name = '"& strProc &"'")
			If colProcessList.count > 0 Then
				ProcessExists = PROCESS_EXISTS
			Else
				ProcessExists = PROCESS_NOT_EXIST
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set colProcessList = Nothing
	End Function
	
	' Returns true if a processes were killed where process strProc is the name of that process(es).
	' ie: strProc is the process name without the .exe "calc" is an example
	Function ProcessKill( strProc )
		ProcessKill = PROCESS_WMI_FAIL
		Dim colProcessList, objProcess
		If Not objWMIService Is Nothing Then
			Set colProcessList = objWMIService.ExecQuery("Select * from Win32_Process WHERE Name = '"& strProc &"'")
			If colProcessList.count > 0 Then
				For Each objProcess In colProcessList
					ProcessKill = objProcess.Terminate()
				Next
			Else
				ProcessKill = PROCESS_NOT_EXIST
			End If 
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set colProcessList = Nothing
	End Function
	
	' Creates a process on the remote host and retuns the pid on success
	' The function returns PROCESS_SUCCESS on success. 
	Function CreateProcess( strExe , strWorkingDir , pid )
        CreateProcess = PROCESS_WMI_FAIL
		Dim objProcess
        If Not objWMIService Is Nothing Then
            Set objProcess = objWMIService.Get("Win32_Process")
            CreateProcess = objProcess.Create(strExe,strWorkingDir,Null,pid)
        End If
    End Function

	' Lists all the processes running where process strProc is the name of that process
	' The function returns PROCESS_SUCCESS on success. If successful arrProc is a list 
	' of processes running on strHost.
	Function ProcessList( byRef arrProc )
		ProcessList = PROCESS_WMI_FAIL
		Dim colProcessList, objProcess, i
		arrProc = Array
		If Not objWMIService Is Nothing Then
			Set colProcessList = objWMIService.ExecQuery("Select * from Win32_Process")
			If colProcessList.count > 0 Then
				For Each objProcess In colProcessList
					i = UBound(arrProc) + 1		
					ReDim Preserve arrProc(i)
					arrProc(i) = objProcess.Name		
				Next
				ProcessList = PROCESS_SUCCESS
			Else
				ProcessList = PROCESS_NOT_EXIST
			End If
		End If
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set colProcessList = Nothing
	End Function
	
	' Function waits for a process strProcName to exit for nMaxWaitTimeSeconds
	' If the function succeeds the return value is true, else false process didn't exit
	' The nMaxWaitTimSeconds should be 5-10 minutes just to take into account slow processes
	Function WaitOnProcessByName( strProcName , nNotExistThreashold , nMaxWaitTimeSeconds )
		WaitOnProcessByName = False
		Dim notExistCount : notExistCount = 0	
		Dim maxCount : maxCount = 0 
		Dim rVal 
		While notExistCount < nNotExistThreashold And maxCount < nMaxWaitTimeSeconds
			' This could return WMI_FAIL
			rVal = ProcessExists( strProcName )
			If rVal = PROCESS_NOT_EXIST Then 
				notExistCount = notExistCount + 1
			Else
				notExistCount = 0
			End If
			maxCount = maxCount + 1
			' Sleep 1 seconds keep looking for setup exec
			WScript.Sleep 1000
		Wend
		' TODO: Perhaps find a better way
		' The process we were watching ended if they are equal
		If notExistCount = nNotExistThreashold Then 
			WaitOnProcessByName = True
		End If		
	End Function
	
	' Members
	Private objWMIService
End Class

'Returns false if no server were found alive, else returns a sorted list of hosts
'outSortedByMs - array sorted fastest response to slowest response Time
Function sort_host_by_ping( ByVal inSeverList , ByRef outSortedByMs )
	On Error Resume Next
	sort_host_by_ping = False
	outLivingSorted = Array
	Dim s, i , j , temp
	For Each s In inSeverList 
		Call objLog.Write("Server: " & s , True )
		Dim bs, br, bl, mi , ma , av
		If Ping( s , bs, br, bl, mi , ma , av ) Then	
			i = UBound(outLivingSorted) + 1		
			ReDim Preserve outLivingSorted(i)
			outLivingSorted(i) = Array(s,av)
			sort_host_by_ping = True ' Success there are servers alive... 
		Else 
			Call objLog.Write(" [Failed]"  & vbCrLf , False)
		End If 
	Next
	'Sort...
	For i = UBound(outLivingSorted) - 1 To 0 Step -1
		For j = 0 To i
			If outLivingSorted(j)(1) > outLivingSorted(j+1)(1) Then
				temp=outLivingSorted(j+1)
				outLivingSorted(j+1)=outLivingSorted(j)
				outLivingSorted(j)=temp
			End If
		Next
	Next 
	'Temp array to store the new pinged and sorted by reponse time...
	Dim temparray
	ReDim temparray(UBound(outLivingSorted))
	For i = 0 To UBound(outLivingSorted) 
		temparray(i) = outLivingSorted(i)(0)
	Next
	outSortedByMs = temparray
End Function 

' Search preferences object is used by multiple search classes and functions
Class std_searchprefs
	Private Sub Class_Initialize()
		m_sExp=".*" 'Match All Regex
		m_bI=False  ' Ignore
		m_bM=False  ' Multi Line
		m_bG=False  ' Global
		m_nD=0  	' Search Depth 0=(Infinite) // Used by FileSearcher
		m_nF=(FS_ATTR_DANY Or FS_ATTR_FANY Or FS_ATTR_FSON Or FS_ATTR_DSON) ' Search enabled for Files\Folders All Attribute Types
	End Sub
	
	Public Property Let SearchExp( sExp )
	m_sExp = sExp
	End Property
	
	Public Property Get SearchExp
	SearchExp = m_sExp
	End Property
	
	Public Property Let Global( bG )
	m_bG = bG
	End Property    
	
	Public Property Get Global
	Global = m_bG
	End Property
	
	Public Property Let NoCase( bI )
	m_bI = bI
	End Property    
	
	Public Property Get NoCase
	NoCase = m_bI
	End Property
	
	Public Property Let MultiLine( bM )
	m_bM = bM
	End Property    
	
	Public Property Get MultiLine
	MultiLine = m_bM
	End Property
	
	Public Property Let Depth( nD )
	m_nD = nD
	End Property  
	
	Public Property Get Depth
	Depth = m_nD
	End Property
	' Needs Full Long Value
	Public Property Let Flags( nF )
	m_nF = CLng(nF)
	End Property  
	' Needs Full Long Value
	Public Property Get Flags
	Flags = CLng(m_nF)
	End Property 
	
	' Private Vars
	Private m_sExp
	Private m_bI
	Private m_bM
	Private m_bG
	Private m_nD
	Private m_nF
End Class

' Registy class allows remote and local registry edits. 		
Class std_registry
	Private Sub Class_Initialize()
		Set objRegistry = Nothing
	End Sub
	
		' Connect to the reg provider for this registy object
	Public Function ConnectProvider32( sComputerName )
		ConnectProvider32 = False
		Set objRegistry = Nothing
		On Error Resume Next
		Dim oLoc : Set oLoc = CreateObject("Wbemscripting.SWbemLocator")
		Dim oCtx : Set oCtx = CreateObject("WbemScripting.SWbemNamedValueSet")
		' Force 64 Bit Registry
		Call oCtx.Add("__ProviderArchitecture", 32 )
		Call oCtx.Add("__RequiredArchitecture", True)
		Dim oSvc : Set oSvc = oLoc.ConnectServer(sComputerName,"root\default","","",,,WBEM_MAX_WAIT,oCtx)
		Set objRegistry = oSvc.Get("StdRegProv")
		If Err.Number = 0 Then			
			ConnectProvider32 = True
		End If
	End Function
	
	' Connect to the reg provider for this registy object
	Public Function ConnectProvider64( sComputerName )
		ConnectProvider64 = False
		Set objRegistry = Nothing
		On Error Resume Next
		Dim oLoc : Set oLoc = CreateObject("Wbemscripting.SWbemLocator")
		Dim oCtx : Set oCtx = CreateObject("WbemScripting.SWbemNamedValueSet")
		' Force 64 Bit Registry
		Call oCtx.Add("__ProviderArchitecture", 64 )
		Call oCtx.Add("__RequiredArchitecture", True)
		Dim oSvc : Set oSvc = oLoc.ConnectServer(sComputerName,"root\default","","",,,WBEM_MAX_WAIT,oCtx)
		Set objRegistry = oSvc.Get("StdRegProv")
		If Err.Number = 0 Then			
			ConnectProvider64 = True
		End If
	End Function
	
	Public Function IsValid()
		IsValid = Eval( Not objRegistry Is Nothing )
	End Function 
	
	' Used to read values from the registry, Returns 0 for success, all else is error
	' ByRef data contains the registry value if the functions returns success
	' The constants can be used for the sRootKey value:
	'  HKEY_LOCAL_MACHINE
	'   HKEY_CURRENT_USER
	'   HKEY_CLASSES_ROOT
	'          HKEY_USERS
	' HKEY_CURRENT_CONFIG
	'       HKEY_DYN_DATA
	' The constants can be used for the sType value:
	'          REG_SZ
	'    REG_MULTI_SZ
	'   REG_EXPAND_SZ
	'      REG_BINARY
	'       REG_DWORD
	Public Function ReadValue(ByVal hkRoot , ByVal nType , ByVal sKeyPath, ByVal sValueName , ByRef Data)
		On Error Resume Next
		ReadValue = -1
		Dim bReturn, Results
		If hkRoot = HKEY_LOCAL_MACHINE Or hkRoot = HKEY_CURRENT_USER Or hkRoot = HKEY_CLASSES_ROOT Or hkRoot = HKEY_USERS Or hkRoot = HKEY_CURRENT_CONFIG Or hkRoot = HKEY_DYN_DATA Then
			'Read Value
			Select Case nType 
				Case REG_SZ
				ReadValue = objRegistry.GetStringValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_MULTI_SZ
				ReadValue = objRegistry.GetMultiStringValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_EXPAND_SZ
				ReadValue = objRegistry.GetExpandedStringValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_BINARY
				ReadValue = objRegistry.GetBinaryValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_DWORD
				ReadValue = objRegistry.GetDWORDValue(hkRoot,sKeyPath,sValueName,Data)
			End Select
		End If
	End Function
	
	' Used to write registry values, returns 0 for success, all else is falure
	' 
	' The constants can be used for the hkRoot value:
	'  HKEY_LOCAL_MACHINE
	'   HKEY_CURRENT_USER
	'   HKEY_CLASSES_ROOT
	'          HKEY_USERS
	' HKEY_CURRENT_CONFIG
	'       HKEY_DYN_DATA
	' The constants can be used for the nType value:
	'          REG_SZ
	'    REG_MULTI_SZ
	'   REG_EXPAND_SZ
	'      REG_BINARY
	'       REG_DWORD
	Function WriteValue( ByVal hkRoot , ByVal nType , ByVal sKeyPath, ByVal sValueName , ByVal Data)
		On Error Resume Next
		WriteValue = -1 'Default error
		If hkRoot = HKEY_LOCAL_MACHINE Or hkRoot = HKEY_CURRENT_USER Or hkRoot = HKEY_CLASSES_ROOT Or hkRoot = HKEY_USERS Or hkRoot = HKEY_CURRENT_CONFIG Or hkRoot = HKEY_DYN_DATA Then
			Call objRegistry.CreateKey( hkRoot , sKeyPath ) 'Create the key if not existing...
			'Read Value
			Select Case nType 
				Case REG_SZ
				WriteValue = objRegistry.SetStringValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_MULTI_SZ
				WriteValue = objRegistry.SetMultiStringValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_EXPAND_SZ
				WriteValue = objRegistry.SetExpandedStringValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_BINARY
				WriteValue = objRegistry.SetBinaryValue(hkRoot,sKeyPath,sValueName,Data)
				Case REG_DWORD
				WriteValue = objRegistry.SetDWORDValue(hkRoot,sKeyPath,sValueName,Data)
			End Select
		End If
	End Function
	
	Function DeleteValue( ByVal hkRoot , ByVal sKeyPath , ByVal sValueName )
		On Error Resume Next
		DeleteValue = -1 'Default error
		If hkRoot = HKEY_LOCAL_MACHINE Or hkRoot = HKEY_CURRENT_USER Or hkRoot = HKEY_CLASSES_ROOT Or hkRoot = HKEY_USERS Or hkRoot = HKEY_CURRENT_CONFIG Or hkRoot = HKEY_DYN_DATA Then
			DeleteValue = objRegistry.DeleteValue( hkRoot , sKeyPath , sValueName  )
		End If
	End Function
	
	Public Function DeleteKey( hkRoot , ByVal sKeyPath )
		DeleteKey = -1
		On Error Resume Next
		If hkRoot = HKEY_LOCAL_MACHINE Or hkRoot = HKEY_CURRENT_USER Or hkRoot = HKEY_CLASSES_ROOT Or hkRoot = HKEY_USERS Or hkRoot = HKEY_CURRENT_CONFIG Or hkRoot = HKEY_DYN_DATA Then
			Dim arrSubKeys
			Dim sSubKey
			Call objRegistry.EnumKey( hkRoot, sKeyPath, arrSubkeys )
			If IsArray(arrSubkeys) Then 
				For Each sSubKey In arrSubkeys 
					Call DeleteKey( hkRoot, sKeyPath & "\" & sSubKey , bForce)
				Next
			End If
			DeleteKey = objRegistry.DeleteKey( hkRoot, sKeyPath )
		End If 
	End Function
	
	Public Function EnumKey( hkRoot , ByVal sKeyPath , sNames )
		EnumKey = -1
		sNames = Array
		If hkRoot = HKEY_LOCAL_MACHINE Or hkRoot = HKEY_CURRENT_USER Or hkRoot = HKEY_CLASSES_ROOT Or hkRoot = HKEY_USERS Or hkRoot = HKEY_CURRENT_CONFIG Or hkRoot = HKEY_DYN_DATA Then
			EnumKey = objRegistry.EnumKey( hkRoot , sKeyPath , sNames )
		End If
	End Function
	
	' Members Variables
	Private objRegistry
End Class
		
' Mail class allows direct email via smtp
Class std_mail
	Private Sub Class_Initialize()
		Set objMessage = CreateObject("CDO.Message") 
		' Since for SMTP mail only
		objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/sendusing") = 2 
		timeout = 60
		port = 25
		enablessl = False	
		server = "localhost"
		setAuthTypeAnonymous
	End Sub
	Public Property Let subject( strSubject )
	objMessage.Subject = strSubject
	End Property
	Public Property Let from( strFrom )
	objMessage.From = strFrom
	End Property
	Public Property Let recipient( strTo )
	objMessage.To = strTo
	End Property
	Public Property Let cc( strCc )
	objMessage.Cc = strCc
	End Property
	Public Property Let bcc( strBcc )
	objMessage.Bcc = strBcc
	End Property
	Public Property Let body( strBody )
	objMessage.TextBody = strBody
	End Property
	Public Property Let server( strServer )
	objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/smtpserver") = strServer
	End Property 
	Public Sub setAuthTypeAnonymous
		objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/smtpauthenticate") = 0 'No Authentication
	End Sub
	Public Sub setAuthTypeBasic
		objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/smtpauthenticate") = 1 'Basic Authentication
	End Sub
	Public Sub setAuthTypeNTLM
		objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/smtpauthenticate") = 2 'NTLM
	End Sub
	Public Property Let user( strUser )
	objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/sendusername") = strUser
	End Property
	Public Property Let password( strPassword )
	objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/sendpassword") = strPassword
	End Property
	Public Property Let port( nPort )
	objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/smtpserverport") = nPort
	End Property
	Public Property Let enablessl( bUseSSL )
	objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/smtpusessl") = bUseSSL
	End Property
	Public Property Let timeout( nTimeout )
	objMessage.Configuration.Fields.Item("http://schemas.microsoft.com/cdo/configuration/smtpconnectiontimeout") = nTimeout
	End Property
	Public Property Let htmlbody( strHTML )
	objMessage.HTMLBody = strHTML
	End Property
	Public Property Let HtmlFileorUrlBody( strHtmlFileorUrl )
	Call objMessage.CreateMHTMLBody( strHtmlFileorUrl )
	End Property
	Public Property Let attachment( strAttachment )
	Call objMessage.AddAttachment( strAttachment )
	End Property
	Sub Send
		objMessage.Configuration.Fields.Update
		objMessage.Send
	End Sub 
	Private objMessage
End Class

	
		
' Logger class allows for multi level logging
' Log Formaters Eg. [%D][%L][%M] 
' 	%D 	- Date
' 	%L 	- Logger Level
' 	%M 	- Message
' Date Formaters
' 	%A 	- Full Weekday Name ( Monday , Tuesday )
' 	%a 	- Abbreviated weekday name ( Mon , Tue )
' 	%B 	- Full month Name ( January , February )
' 	%b 	- Abbreviated month name ( Jan , Feb )
' 	%Y	- 4 Digit Year
' 	%y	- 2 Digit Year
' 	%M	- 2 Digit Month (Padded 0)
' 	%m	- Digit Month
' 	%D 	- 2 Digit Day (Padded 0)
' 	%d 	- Digit Day
' 	%H 	- 2 Digit Hour 24hr (Padded 0)
' 	%h 	- Digit Hour 24hr
' 	%I 	- 2 Digit Hour 12hr (Padded 0)
' 	%i	- Digit Hour 12hr
' 	%N 	- 2 Digit Minute (Padded 0)
' 	%n 	- Digit Minute
' 	%S 	- 2 Digit Second (Padded 0)
' 	%s 	- Digit Second
' 	%q	- Quarter
' 	%J 	- Day of Year (Padded 0)
' 	%j 	- Day of Year (Padded 0)
' 	%w 	- Day of Week
' 	%U 	- Week of Year (Padded 0)
' 	%u 	- Week of Year
' 	%P 	- Meridiem (Upper Case)
' 	%p 	- Meridiem (Lower Case)
Class std_logfile
	Private Sub Class_Initialize()
		' Will not split entries to display...
		bConsole = False
		sDFmt="%Y%M%D%H%N%S" ' Base Date Format
		sLFmt="[%D][%L][%M]" & vbCrLf ' Base Log Format
		log_level = LOG_ALL
	End Sub
	
	Private Function GetLoggerLevelText( level )
		GetLoggerLevelText = "UNK"
		Select Case level
			Case LOG_SEVERE
			GetLoggerLevelText = "SEV"
			Case LOG_ERROR
			GetLoggerLevelText = "ERR"
			Case LOG_WARNING
			GetLoggerLevelText = "WRN"
			Case LOG_DEBUG
			GetLoggerLevelText = "DBG"
			Case LOG_INFO
			GetLoggerLevelText = "INF"
			Case LOG_CONFIG
			GetLoggerLevelText = "CFG"
		End Select
	End Function 
	
	' Writes a log entry
	Public Sub Write( ByVal message , level )	
		If (log_level And level) Then
			Dim tokens : tokens = Array( Array("%D",strftime( sDFmt , Now )), Array("%L",GetLoggerLevelText(level)) , Array("%M",message) )			
			Dim line : line = sLFmt
			Dim element
			For Each element In tokens
				line = Replace( line , element(0) , element(1) ,1,Len(line), vbBinaryCompare )
			Next
			oFile.Write( line )
			On Error Resume Next ' The following line only works if console is present
			If bConsole Then WScript.StdOut.Write( line )
		End If
	End Sub
	
	Public Sub WriteFormatted( ByVal sFmt , ByVal message )
		Dim tokens : tokens = Array( Array("%D",strftime( sDFmt , Now )), Array("%L",GetLoggerLevelText(LOG_INFO)) , Array("%M",message) )			
		Dim line : line = sFmt
		Dim element
		For Each element In tokens
			line = Replace( line , element(0) , element(1) ,1,Len(line), vbBinaryCompare )
		Next
		oFile.Write( line )
		On Error Resume Next ' The following line only works if console is present
		If bConsole Then WScript.StdOut.Write( line )
	End Sub
	
	Public Sub WriteFormattedLevel( ByVal sFmt , ByVal message , level )
		If (log_level And level) Then
			Dim tokens : tokens = Array( Array("%D",strftime( sDFmt , Now )), Array("%L",GetLoggerLevelText(level)) , Array("%M",message) )			
			Dim line : line = sFmt
			Dim element
			For Each element In tokens
				line = Replace( line , element(0) , element(1) ,1,Len(line), vbBinaryCompare )
			Next
			oFile.Write( line )
			On Error Resume Next ' The following line only works if console is present
			If bConsole Then WScript.StdOut.Write( line )
		End If
	End Sub
	
	Public Sub WriteRaw( ByVal message )
		oFile.Write( message )
		On Error Resume Next ' The following line only works if console is present
		If bConsole Then WScript.StdOut.Write( message )
	End Sub
	
	' Opens or Creates a log file
	Public Function Open( strFile , bOverwrite )
		Open=False
		Dim open_mode : open_mode = ForAppending
		If bOverwrite Then open_mode = ForWriting
		Err.Clear
		Set oFile = objFSO.OpenTextFile( strFile , open_mode , True , TriStateAscii )
		If Err.Number = 0 Then Open=True
	End Function
	
	' Closes the open file object
	Public Sub Close()
		oFile.Close()
		' Setting to Nothing is not needed, but we are doing it just to be sure
		Set oFile = Nothing
	End Sub
	
	' Sets the date format string
	Public Property Let DateFormat( sDateFormat )
	sDFmt = sDateFormat
	End Property
	
	' Returns the current date format string
	Public Property Get DateFormat()
	DateFormat = sDFmt
	End Property
	
	' Sets the date format string
	Public Property Let LogFormat( sLogFormat )
	sLFmt = sLogFormat
	End Property
	
	' Returns the current date format string
	Public Property Get LogFormat()
	LogFormat = sLFmt
	End Property
	
	' Allows enables splitting output to log and console
	Public Property Let EnableOutputToConsole( bEnabled ) 
	bConsole = bEnabled
	End Property 
	' Allows user to set the logger levels
	Public Property Let LoggerLevel( level ) 
	log_level = level
	End Property 
	' Members
	Private bConsole
	Private oFile
	Private sDFmt
	Private sLFmt
	Private log_level
End Class	
		
' Data type to deal with data without need for Set when data is of object type
Class std_auto
	Private Sub Class_Initialize()
		Set m_oData=Nothing
	End Sub
	
	Private sub Class_Terminate() 
		If IsObject( m_oData ) Then Set m_oData=Nothing
	End Sub
	
	Public Property Let Value( oVal )
	If IsObject( oVal ) Then
		If IsObject( m_oData ) Then Set m_oData = Nothing
		Set m_oData = oVal
	Else
		m_oData = oVal
	End If
	End Property
	
	Public Default Property Get Value()
	If IsObject( m_oData ) Then
		Set Value = m_oData
	Else
		Value = m_oData
	End If
	
	End Property
	' Private Data Value
	Private m_oData
End Class

' For single link
Class std_node
	Private Sub Class_Initialize()
		Set m_oData=Nothing
		Set m_oNext=Nothing
	End Sub
	
	Private sub Class_Terminate()
		If IsObject( m_oData ) Then Set m_oData=Nothing
		Set m_oNext=Nothing
	End Sub
	
	' Setter for data which can be an object so new will be used
	Public Property Let Data( oData )
		If IsObject( oData ) Then
			If IsObject( m_oData ) Then Set m_oData = Nothing
			Set m_oData = oData
		Else
			m_oData = oData
		End If
	End Property
	
	' Getter for data which can be an object so new will be used
	Public Property Get Data()
		If IsObject( m_oData ) Then
			Set Data = m_oData
		Else
			Data = m_oData
		End If
	End Property
	
	Public Property Let ObjectNext( oNext )
		Set m_oNext = oNext
	End Property
	
	Public Property Get ObjectNext()
		Set ObjectNext = m_oNext
	End Property
	
	' Members
	Private m_oData
	Private m_oNext
End Class
		
' Double Link Node
Class std_dnode
	Private Sub Class_Initialize()
		Set m_oData=Nothing
		Set m_oNext=Nothing
		Set m_oPrev=Nothing
	End Sub
	
	Private sub Class_Terminate()
		If IsObject( m_oData ) Then Set m_oData=Nothing
		Set m_oNext=Nothing
		Set m_oPrev=Nothing 
	End Sub
	
	' Setter for data which can be an object so new will be used
	Public Property Let Data( oData )
		If IsObject( oData ) Then
			If IsObject( m_oData ) Then Set m_oData = Nothing
			Set m_oData = oData
		Else
			m_oData = oData
		End If
	End Property
	
	' Getter for data which can be an object so new will be used
	Public Property Get Data()
		If IsObject( m_oData ) Then
			Set Data = m_oData
		Else
			Data = m_oData
		End If
	End Property
	
	Public Property Let ObjectNext( oNext )
		Set m_oNext = oNext
	End Property
	
	Public Property Get ObjectNext()
		Set ObjectNext = m_oNext
	End Property

	Public Property Let ObjectPrev( oPrev )
		Set m_oPrev = oPrev
	End Property
	
	Public Property Get ObjectPrev()
		Set ObjectPrev = m_oPrev
	End Property
				
	' Members
	Private m_oData
	Private m_oNext
	Private m_oPrev
End Class
		
' Custom Data Types Tree, Stack, Queue, Deque

Class std_stack
	Private Sub Class_Initialize()
		Set m_oHead=Nothing
		m_nLength = 0
	End Sub			
	
	Private sub Class_Terminate() 
		Set m_oHead=Nothing
	End Sub
	
	Private Function make_node( oData )
		Dim o : Set o = New std_node
		If Not o Is Nothing Then
			o.Data = oData
			m_nLength = m_nLength + 1
		End If
		' Return our newly created object
		Set make_node = o
	End Function
	
	Public Function push( oData )
		If m_oHead Is Nothing Then 
			Set m_oHead = make_node( oData )
		Else
			Dim oTemp : Set oTemp = make_node( oData ) 
			oTemp.ObjectNext = m_oHead
			Set m_oHead = oTemp
		End If
	End Function
		
	Public Function pop( )
		Set Pop = Nothing
		If Not m_oHead Is Nothing Then	
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oHead.Data
			Set m_oHead = m_oHead.ObjectNext
			m_nLength = m_nLength - 1
			Set Pop = oTemp
    	End If
	End Function
	
	Public Function peek( )
		Set Peek = Nothing
		If Not m_oHead Is Nothing Then 
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oHead.Data
			Set Peek = oTemp
    	End If
	End Function
	
	Public Function get_iterator()
		Dim i : Set i = New std_iterator
		Call i( m_oHead , False )
		Set get_iterator = i
	End Function
	
	' Size of the stack
	Public Property Get Size()
		Size = m_nLength
	End Property
	
	Public Property Get has_items()
		has_items = Eval(m_nLength > 0)
	End Property
	
	' Private Data
	Private m_nLength
	Private m_oHead
	'Private m_fnComp ' Unused
End Class

Class std_queue
	Private Sub Class_Initialize()
		Set m_oHead=Nothing
		Set m_oTail=Nothing
		m_nLength = 0
	End Sub
	
	Private sub Class_Terminate()
		Set m_oHead=Nothing
		Set m_oTail=Nothing
	End Sub	
	
	Private Function make_node( oData )
		Dim o : Set o = New std_node
		If Not o Is Nothing Then
			o.Data = oData
			m_nLength = m_nLength + 1
		End If
		' Return our newly created object
		Set make_node = o
	End Function

	Public Function enqueue( oData )
		If m_oHead Is Nothing And m_oTail Is Nothing Then 
			Set m_oHead = make_node( oData )
			Set m_oTail = m_oHead
		Else
			Dim oTemp : Set oTemp = make_node( oData )
			If m_oHead Is m_oTail Then
				Set m_oTail = oTemp
				m_oHead.ObjectNext = m_oTail
			Else
				m_oTail.ObjectNext = oTemp
				Set m_oTail = m_oTail.ObjectNext
			End If
		End If
	End Function
		
	Public Function dequeue( )
		Set dequeue = Nothing
		If Not m_oHead Is Nothing And Not m_oTail Is Nothing Then
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oHead.Data
			m_nLength = m_nLength - 1
			If m_oHead Is m_oTail Then
				Set m_oHead = Nothing
				Set m_oTail = m_oHead
			Else
				Set m_oHead = m_oHead.ObjectNext
    		End If
    		Set dequeue = oTemp
    	End If
	End Function
	
	Public Function peek( )
		Set Peek = Nothing
		If Not m_oHead Is Nothing Then 
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oHead.Data
			Set Peek = oTemp
    	End If
	End Function
	
	Public Function get_iterator()
		Dim i : Set i = New std_iterator
		Call i( m_oHead , False )
		Set get_iterator = i
	End Function
	
	' Size of the stack
	Public Property Get size()
		Size = m_nLength
	End Property

	Public Property Get has_items()
		has_items = Eval(m_nLength > 0)
	End Property
	
	' Private Data
	Private m_nLength
	Private m_oHead
	Private m_oTail
End Class

Class std_deque
	Private Sub Class_Initialize()
		Set m_oHead=Nothing
		Set m_oTail=Nothing
		m_nLength = 0
	End Sub

	Private sub Class_Terminate()
		Set m_oHead=Nothing
		Set m_oTail=Nothing
	End Sub	

	Private Function make_node( oData )
		Dim o : Set o = New std_dnode
		If Not o Is Nothing Then
			o.Data = oData
			m_nLength = m_nLength + 1
		End If
		' Return our newly created object
		Set make_node = o
	End Function

	Public Function push_front( oData )
		If m_oHead Is Nothing And m_oTail Is Nothing Then 
			Set m_oHead = make_node( oData )
			Set m_oTail = m_oHead
		Else
			Dim oTemp : Set oTemp = make_node( oData )
			If m_oHead Is m_oTail Then
				Set m_oHead = oTemp
				m_oHead.ObjectNext = m_oTail
				m_oTail.ObjectPrev = m_oHead
			Else
				m_oHead.ObjectPrev = oTemp
				oTemp.ObjectNext = m_oHead
				Set m_oHead = oTemp
			End If
		End If
	End Function

	Public Function push_back( oData )
		If m_oHead Is Nothing And m_oTail Is Nothing Then 
			Set m_oHead = make_node( oData )
			Set m_oTail = m_oHead
		Else
			Dim oTemp : Set oTemp = make_node( oData )
			If m_oHead Is m_oTail Then
				Set m_oTail = oTemp
				m_oHead.ObjectNext = m_oTail
				m_oTail.ObjectPrev = m_oHead
			Else
				m_oTail.ObjectNext = oTemp
				oTemp.ObjectPrev = m_oTail
				Set m_oTail = m_oTail.ObjectNext
			End If
		End If
	End Function
		
	Public Function pop_front( )
		If m_oHead Is Nothing And m_oTail Is Nothing Then 
			Set dequeue = Nothing
		Else
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oHead.Data
			Set pop_front = oTemp
			m_nLength = m_nLength - 1
			If m_oHead Is m_oTail Then
				Set m_oHead = Nothing
				Set m_oTail = m_oHead
			Else
				Set m_oHead = m_oHead.ObjectNext
				m_oHead.ObjectPrev = Nothing						
    		End If
    	End If
	End Function
	
	Public Function pop_back( )
		If m_oHead Is Nothing And m_oTail Is Nothing Then 
			Set dequeue = Nothing
		Else
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oTail.Data
			Set pop_back = oTemp
			m_nLength = m_nLength - 1
			If m_oHead Is m_oTail Then
				Set m_oHead = Nothing
				Set m_oTail = m_oHead
			Else
				Set m_oTail = m_oTail.ObjectPrev
				m_oTail.ObjectNext = Nothing						
    		End If
    	End If
	End Function
	
	Public Function peek_front( )
		Set peek_front = Nothing
		If Not m_oHead Is Nothing Then 
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oHead.Data
			Set peek_front = oTemp
    	End If
	End Function

	Public Function peek_back( )
		Set peek_back = Nothing
		If Not m_oTail Is Nothing Then 
			Dim oTemp : Set oTemp = New std_auto
			oTemp.Value = m_oTail.Data
			Set peek_back = oTemp
    	End If
	End Function

	Public Function get_iterator()
		Dim i : Set i = New std_iterator
		Call i( m_oHead , False )
		Set get_iterator = i
	End Function
	
	Public Function get_rev_iterator()
		Dim i : Set i = New std_iterator
		Call i( m_oTail , True )
		Set get_rev_iterator = i
	End Function
	
	' Size of the stack
	Public Property Get size()
		Size = m_nLength
	End Property

	Public Property Get has_items()
		has_items = Eval(m_nLength > 0)
	End Property
	
	Private m_nLength
	Private m_oHead
	Public m_oTail
End Class
			
				
Class std_rb_node
	Private Sub Class_Initialize()
		m_bRed=True
		Set m_oData=Nothing
		Set m_left=Nothing
		Set m_right=Nothing
	End Sub
	
	Private sub Class_Terminate()
		If IsObject( m_oData ) Then Set m_oData=Nothing
		Set m_left=Nothing
		Set m_right=Nothing
	End Sub
	
	Public Property Let Red( bRed )
	m_bRed = bRed
	End Property
	
	Public Property Get Red()
	Red = m_bRed
	End Property
	
	' Setter for data which can be an object so new will be used
	Public Property Let Data( oData )
	If IsObject( oData ) Then
		If IsObject( m_oData ) Then Set m_oData = Nothing
		Set m_oData = oData
	Else
		m_oData = oData
	End If
	End Property
	
	' Getter for data which can be an object so new will be used
	Public Property Get Data()
	If IsObject( m_oData ) Then
		Set Data = m_oData
	Else
		Data = m_oData
	End If
	End Property
	
	Public Property Let ObjectLeft( oLeft )
	Set m_left = oLeft
	End Property
	
	Public Property Get ObjectLeft()
	Set ObjectLeft = m_left
	End Property
	
	Public Property Let ObjectRight( oRight )
	Set m_right = oRight
	End Property
	
	Public Property Get ObjectRight()
	Set ObjectRight = m_right
	End Property
	
	' Private Data
	Private m_bRed
	Private m_oData
	Private m_left
	Private m_right
End Class

Class std_rb_tree
	Private Sub Class_Initialize()
		Set m_fnSort=Nothing
		Set m_rbRoot=Nothing
		m_nLength=0
	End Sub
	
	Private sub Class_Terminate() 
		Set m_fnSort=Nothing
		Set m_rbRoot=Nothing
	End Sub
	
	' Register callback function which returns less, greater, or equal for your object
	' Callback must be defined as SomeFunction( lhs , rhs )
	' See the following example for usage:
	'
	' Function mySortFunction( lhs , rhs )
	' 	if lhs < rhs then 
	'	 	mySortFunction = -1
	'	 elseif lhs > then
	'	 	mySortFunction = 1
	'	 else
	' 		mySortFunction = 0
	' 	End If
	' End Function
	'
	' Dim map : Set map = New RBTree
	' RBTree.RegisterCallback( GetRef("mySortFunction") )
	'
	Public Function RegisterCallback( byRef fnSort )
		If Not IsObject( fnSort ) Then Call Err.Raise( 424, "RegisterCallback", "Function expect an argument function pointer or Nothing" )
		Set m_fnSort = fnSort
	End Function    
	
	' Private Functions
	Public Function is_red( oNode )
		If Not oNode Is Nothing Then is_red=oNode.Red Else is_red=False
	End Function
	
	Private Function make_node( oData )
		Dim o : Set o = New std_rb_node
		If Not o Is Nothing Then
			o.Data = oData
			m_nLength = m_nLength + 1
		End If
		' Return our newly created object
		Set make_node = o
	End Function
	
	Private Function pivot_right( oNode )
		Dim oTemp : Set oTemp = oNode.ObjectLeft
		oNode.ObjectLeft = oTemp.ObjectRight 		
		oTemp.ObjectRight = oNode
		oNode.Red = True
		oTemp.Red = False
		Set pivot_right = oTemp
	End Function	
	
	Private Function pivot_left( oNode )
		Dim oTemp : Set oTemp = oNode.ObjectRight 
		oNode.ObjectRight = oTemp.ObjectLeft
		oTemp.ObjectLeft = oNode
		oNode.Red = True
		oTemp.Red = False
		Set pivot_left = oTemp
	End Function	
	
	' Wikipedia - Tree Rotate
	' There are also double rotations, which are combinations of left and right rotations. 
	' A double left rotation at X can be defined to be 
	' a right rotation at the right child of X followed by 
	' a left rotation at X;
	' similarly, a double right rotation at X can be defined to be a left 
	' rotation at the left child of X followed by a right rotation at X.
	Private Function pivot_2x_right( oNode )
		oNode.ObjectLeft = pivot_left( oNode.ObjectLeft )
		Set pivot_2x_right = pivot_right( oNode )
	End Function	
	
	Private Function pivot_2x_left( oNode )
		oNode.ObjectRight = pivot_right( oNode.ObjectRight )
		Set pivot_2x_left = pivot_left( oNode )
	End Function	
	
	Public Function validate()
		validate_structure(m_rbRoot)
	End Function
	
	Private Function validate_structure( oNode )
		Dim lh, rh
		
		If oNode Is Nothing Then
			validate_structure=1
		Else
			Dim ln , rn
			Set ln = oNode.ObjectLeft
			Set rn = oNode.ObjectRight
			
			If is_red( oNode ) Then 
				If is_red( ln ) Or is_red ( rn ) Then
					WScript.Echo( "Red violation" )
					Exit Function
				End If
			End If 
			
			lh = validate_structure( ln )
			rh = validate_structure( rn )
			
			' Setup variables to be successfull
			Dim lc : lc = -1 ' Less
			Dim rc : rc = 1 ' Greater
			
			If Not ln Is Nothing Then
				lc = m_fnSort( ln.Data , oNode.Data )
			End If
			
			If Not rn Is Nothing Then
				rc = m_fnSort( rn.Data , oNode.Data )
			End If
			
			If lc >= 0 Or rc <= 0 Then
				WScript.Echo( "Binary Tree Violation" )
				validate_structure=0
				Exit Function
			End If
			
			'Black height mismatch
			If lh <> 0 And rh <> 0  And lh <> rh Then
				WScript.Echo( "Black Violation" )
				validate_structure=0
				Exit Function
			End If
			
			'Only count black links */
			If lh <> 0 And rh <> 0 Then
				If is_red( oNode ) Then validate_structure=lh Else validate_structure=lh + 1
			Else
				validate_structure=0
			End If
		End If 			
	End Function 
	
	Private Function insert_rebalance( oNode, oData )
		If oNode Is Nothing Then
			Set oNode = make_node( oData )
		Else
			Select Case m_fnSort( oNode.Data , oData )
			Case 0
				oNode.Data = oData  'Overwrite
			Case 1
				oNode.ObjectLeft = insert_rebalance( oNode.ObjectLeft ,  oData )
				If is_red( oNode.ObjectLeft ) Then
					If is_red( oNode.ObjectRight ) Then
						'Case 1 Recolor
						oNode.Red = True
						oNode.ObjectLeft.Red = False
						oNode.ObjectRight.Red = False
					Else
						If is_red( oNode.ObjectLeft.ObjectLeft ) Then
							Set oNode = pivot_right( oNode )
						ElseIf is_red( oNode.ObjectLeft.ObjectRight ) Then
							Set oNode = pivot_2x_right( oNode )
						End If
					End If
				End If
			Case -1
				oNode.ObjectRight = insert_rebalance( oNode.ObjectRight ,  oData )
				If is_red( oNode.ObjectRight ) Then
					If is_red( oNode.ObjectLeft ) Then
						'Case 1 Recolor
						oNode.Red = True
						oNode.ObjectLeft.Red = False
						oNode.ObjectRight.Red = False
					Else
						If is_red( oNode.ObjectRight.ObjectRight ) Then
							Set oNode = pivot_left( oNode )
						ElseIf is_red( oNode.ObjectRight.ObjectLeft ) Then
							Set oNode = pivot_2x_left( oNode )
						End If
					End If	
				End If
			End Select
		End If 
		Set insert_rebalance = oNode
	End Function
	
	Function remove_balance_left( oNode , done )
		Dim p : Set p = oNode
		Dim s : Set s = oNode.ObjectRight
		
		If is_red ( s ) Then
			Set oNode = pivot_left( oNode )
			Set s = p.ObjectRight
		End If
		
		If Not s Is Nothing Then 
			If Not is_red ( s.ObjectRight ) And Not is_red ( s.ObjectLeft ) Then
				If is_red ( p ) Then
					done = True
				End If		
				p.red = False
				s.red = True
			Else
				Dim keep : keep = p.red
				Dim new_root : new_root = Eval(oNode Is p)
				
'				If new_root Then				
'					WScript.Echo "Same"
'				Else
'					WScript.Echo "Diff"
'				End If
				
				If is_red( s.ObjectRight ) Then 
					Set p = pivot_left( p )
				Else
					Set p = pivot_2x_left( p )
				End If
				p.red = keep
				p.ObjectRight.red = False
				p.ObjectLeft.red = False
				
				If new_root Then				
					Set oNode = p
				Else
					oNode.ObjectLeft = p
				End If
				done = True
			End If
		End If
		Set remove_balance_left=oNode
	End Function
	
	
	Function remove_balance_right( oNode , done )
		Dim p : Set p = oNode
		Dim s : Set s = oNode.ObjectLeft
		
		If is_red ( s ) Then
			Set oNode = pivot_right( oNode )
			Set s = p.ObjectLeft
		End If
		
		If Not s Is Nothing Then 
			If Not is_red ( s.ObjectRight ) And Not is_red ( s.ObjectLeft ) Then
				If is_red ( p ) Then
					done = True
				End If		
				p.red = False
				s.red = True
			Else
				Dim keep : keep = p.red
				Dim new_root : new_root = Eval(oNode Is p)
				If is_red( s.ObjectLeft ) Then 
					Set p = pivot_right( p )
				Else
					Set p = pivot_2x_right( p )
				End If
				p.red = keep
				p.ObjectRight.red = False
				p.ObjectLeft.red = False
				If new_root Then
					Set oNode = p
				Else
					oNode.ObjectRight = p
				End If
				done = True
			End If
		End If
		Set remove_balance_right=oNode
	End Function
	
	Private Function remove_rebalance( oNode , oData , ByRef done )
		If oNode Is Nothing Then
			done = True
		Else
			Select Case m_fnSort( oNode.Data , oData )
				Case 0
					If oNode.ObjectLeft Is Nothing Or oNode.ObjectRight Is Nothing Then
						Dim rval
						
						If oNode.ObjectLeft Is Nothing Then 
							Set rval = oNode.ObjectRight 
						Else 
							Set rval = oNode.ObjectLeft
						End If
						
						If is_red( oNode ) Then
							done = True
						ElseIf is_red( rval ) Then
							rval.red = 0
							done = True
						End If
						
						Set oNode = Nothing
						
						m_nLength = m_nLength-1
						
						Set remove_rebalance = rval
						
						Exit Function
					Else
						Dim temp : Set temp = oNode.ObjectLeft
						While Not temp.ObjectRight Is Nothing
							Set temp = temp.ObjectRight
						Wend
						oNode.data = temp.Data
						If IsObject( oData ) Then 
							Set oData = temp.Data
						Else
							oData = temp.Data
						End If
					End If
				Case 1
					oNode.ObjectLeft = remove_rebalance( oNode.ObjectLeft , oData , done )
					If Not done Then Set oNode = remove_balance_left( oNode , done )
				Case -1
					oNode.ObjectRight = remove_rebalance( oNode.ObjectRight , oData , done )
					If Not done Then Set oNode = remove_balance_right( oNode , done )
			End Select
		End If
		Set remove_rebalance = oNode
	End Function
	
	Private Function find_data( oNode , oData )
		If oNode Is Nothing Then
			Set find_data = Nothing
		Else
			Select Case m_fnSort( oNode.Data , oData )
				Case 0
				Dim ref : Set ref = New std_auto
				ref.Value = oNode.Data
				Set find_data = ref
				Case 1
				Set find_data = find_data( oNode.ObjectLeft , oData )	
				Case -1
				Set find_data = find_data( oNode.ObjectRight , oData )
			End Select 			
		End If
	End Function
	
	' Returns std_rb_iterator
	Public Function get_iterator()
		 Dim i : Set i = New std_iterator
		 Call i( m_rbRoot , False )
		 Set get_iterator = i
	End Function

	' Insert an object
	Public Function insert( oData )
		Set m_rbRoot = insert_rebalance( m_rbRoot , oData )
		m_rbRoot.Red = False
	End Function
	
	' Remove a value by key
	Public Function remove( oData )
		Dim result : result = False
		Set m_rbRoot = remove_rebalance( m_rbRoot , oData , result )
		If Not m_rbRoot Is Nothing Then m_rbRoot.Red = False
		remove = result
	End Function
	
	' Remove all items in the tree
	Public Sub clear()
		Set m_rbRoot = Nothing
		m_nLength = 0
	End Sub
	
	' Find an object using the key
	Public Function find( oData )
		Set find = find_data( m_rbRoot , oData )
	End Function
	
	' Size of tree
	Public Property Get Size()
	Size = m_nLength
	End Property

	Public Property Get has_items()
		has_items = Eval(m_nLength > 0)
	End Property
	
	' Member Variables
	Private m_nLength
	Private m_rbRoot ' Points to the root element of the data structure
	Private m_fnSort ' Will be a custom sort function which can be registered
End Class

Class std_iterator
	Private Sub Class_Initialize()
		Set m_o = Nothing ' Current Node or Queue for tree
		m_breverse = False ' Forward iterator (default)
		m_btree = False ' Not a tree iterator by default
	End Sub

	Private sub Class_Terminate()
		Set m_o = Nothing
	End Sub
	
	Public Default Function Init( oData , bReverse )
		Set m_o = Nothing
		m_btree = False
		m_breverse  = False
		Select Case TypeName( oData )
			Case "std_node"
					Set m_o = oData
			Case "std_dnode"
					m_breverse  = bReverse ' Since dnode we can have direction
					Set m_o = oData
			Case "std_rb_node"
					m_btree = True
					Set m_o = New std_queue
					m_o.enqueue( oData )
		End Select
	End Function
	
	' Returns true if more items exist
	Public Function more_data()
		If m_btree Then
			more_data = Eval(m_o.size > 0)
		Else
			more_data = Eval(Not m_o Is Nothing)
		End If
	End Function
	
	' Returns Nothing or std_auto
	Public Function get_next()
		Set get_next = Nothing
		If m_btree Then
			If m_o.size > 0 Then
				' Get std_auto from dequeue
				Dim c : Set c = m_o.dequeue()
				If Not c.Value Is Nothing Then
					If Not c.Value.ObjectLeft Is Nothing Then m_o.enqueue( c.Value.ObjectLeft )
					If Not c.Value.ObjectRight Is Nothing Then m_o.enqueue( c.Value.ObjectRight )
					Set get_next = New std_auto
					get_next.Value = c.Value.Data
				End If
			End If 
		Else
			If m_breverse Then
				If Not m_o Is Nothing Then
					Set get_next = New std_auto
					get_next.Value = m_o.Data
					Set m_o = m_o.ObjectPrev
    			End If
			Else
				If Not m_o Is Nothing Then
					Set get_next = New std_auto
					get_next.Value = m_o.Data
					Set m_o = m_o.ObjectNext    		
    			End If
    		End If
    	End If
	End Function
	
	Private m_o			' Current Node
	Private m_breverse	' Forward or Reverse iterator
	Private m_btree		' Tree or Linked list
End Class

' RC4 Encryption Class
Class std_rc4
	Private m_sbox(255)
	Private m_key(255)
	
	'Helper Function... 
	Private Sub Init( key )
		Dim t
		Dim a
		Dim b : b = 0 
		Dim n : n = Len( key )
		For a = 0 To 255
			m_key(a) = asc(Mid(key, (a mod n)+1, 1))
			m_sbox(a) = a
		Next
		For a = 0 To 255
			b = (b + m_sbox(a) + m_key(a)) Mod 256
			t = m_sbox(a)
			m_sbox(a) = m_sbox(b)
			m_sbox(b) = t
		Next
	End Sub
	
	' Encrypts or Decrypts
	Private Function EnDeCrypt(plaintxt, key)
		Dim temp
		dim a
		dim i : i = 0
		dim j : j = 0
		dim k
		dim cipherby
		dim cipher
		Init key
		For a = 1 To Len(plaintxt)
			i = (i + 1) Mod 256
			j = (j + m_sbox(i)) Mod 256
			temp = m_sbox(i)
			m_sbox(i) = m_sbox(j)
			m_sbox(j) = temp
			k = m_sbox((m_sbox(i) + m_sbox(j)) Mod 256)
			cipherby = Asc(Mid(plaintxt, a, 1)) Xor k
			cipher = cipher & Chr(cipherby)
		Next
		EnDeCrypt = cipher
	End Function
	
	'Decodes the HEXcrypt to text
	Public Function DecodeHexString( str , key )
		Dim outStr : outStr = ""
		Dim pos
		For pos = 1 To Len(str) Step 2
				outStr = outStr + Chr(hex2dec( Mid(str , pos , 2 ) ))
		Next
		DecodeHexString = EnDeCrypt( outStr , key )
	End Function 
	
	'Convert a String to HEXCrypt
	Public Function EncodeHexString( str  , key)
		str = EnDeCrypt( str , key )
		Dim outStr : outStr = ""
		Dim pos
		For pos = 1 To Len(str) 
				Dim ascstr : ascstr = CStr(Hex(Asc(Mid( str , pos , 1 ))))
				If Len(ascstr) = 1 Then ascstr = "0" + ascstr
			outStr = outStr + ascstr
		Next
		EncodeHexString = outStr
	End Function
End Class

' END OF TEMPLATE FUNCTIONS
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************


' Usage ( CAN BE MODIFIED GENERIC )
' ************************************************************************************************
' ************************************************************************************************
' Displays usage to the user
Sub Usage()
	Call WScript.Echo( vbCrLf & "Usage: " & WScript.ScriptName & " (-i|-u)" & vbCrLf &_
	"-i" & vbTab & "- Install" & vbCrLf &_
	"-u" & vbTab & "- Uninstall" & vbCrLf &_
	vbCrLf & "All rights reserved." )
	
	' Pause output if shell processed
	If IsCScript() Then 
		WScript.Echo "Press any key to exit..." 
		WScript.StdIn.Read(1)
	End If 
	WScript.Quit 1
End Sub

' ************************************************************************************************
' ************************************************************************************************
' ************************************************************************************************
' Script entry point here ( Main Should Return Int )
WScript.Quit Main()
' ************************************************************************************************

'*************************************************************************************************
'	DEFINE CUSTOM CONSTANTS HERE OR GLOBAL VARIABLES
'
'	ALL CODE BELOW THIS POINT IS NON TEMPLATE
'
'*************************************************************************************************
	
' Standard Main Function To Demonstrate Entry Point and Arg parsing
Function Main()
	'Call Usage()
	Call TestFeatures()		
End Function

'RB Tree Custom Sort Function
Function fnSort( lhs , rhs )	
	Call objLog.Write( "fnSort: " & lhs & " <-> " & rhs & " = "& StrComp(lhs,rhs,vbTextCompare) , LOG_INFO ) 
	fnSort = StrComp(lhs,rhs,vbTextCompare)
End Function

'RB Tree Custom Sort Function
Function fnMapSort( lhs , rhs )	
	fnMapSort = StrComp(lhs.key,rhs.key,vbTextCompare)
End Function

' Call back for std_fs_search
Function myFileSearchCallback( oFile , byRef oData )
	myFileSearchCallback=False ' Keep searching
	Call objLog.Write( "myFileSearchCallback: " & "---------------------------------------------" , LOG_INFO )
	Call objLog.Write( "myFileSearchCallback: " & "ObjectType: " & TypeName(oFile) , LOG_INFO )
	Call objLog.Write( "myFileSearchCallback: " & "ObjectAttr: " & oFile.attributes , LOG_INFO )
	Call objLog.Write( "myFileSearchCallback: " & "ObjectPath: " & oFile.Path , LOG_INFO )
	
	' Let's stop if we find the needle.txt file just for fun
	If StrComp(TypeName(oFile),"File",vbTextCompare) = 0 And StrComp(oFile.Name,"needle.txt",vbTextCompare) = 0 Then
		Call objLog.Write(  "myFileSearchCallback: " & "Callback function found file and issuing an abort" , LOG_INFO )
		myFileSearchCallback=True ' Stop searching
		oData = oFile.Path ' Save host file location.
	End If
	Call objLog.Write( "myFileSearchCallback: " & "---------------------------------------------" ,LOG_INFO)
End Function
		
' Function to test various features
Function TestFeatures()

	Dim ping_lookup_host : ping_lookup_host = "google.com"
	Dim auto ' Auto
	Dim itr  ' Iterator
 
	' Show console logging also
	objLog.EnableOutputToConsole = True
	' Uncomment to modify date\logging format
	'objLog.DateFormat =  "%M/%D/%Y %I:%N:%S %P"
	'objLog.LogFormat =  "%D,%L,%M" & vbCrLf
	
	' Test log file
	Call objLog.Open(LOG_FILE , True)	

'************************************************************************************************
'                                 std_rb_tree tests start
'************************************************************************************************
	
	' RBTree
	Call objLog.Write( "Testing RB Tree Insert" , LOG_INFO )
	Dim rb : Set rb = New std_rb_tree	

	' Register the sorting function for our datatype
	rb.RegisterCallback( GetRef("fnSort") )
	
	rb.insert("L")
	rb.insert("M")
	rb.insert("N")
	rb.insert("O")
	rb.insert("P")
	rb.insert("Q")
	rb.insert("R")	
	rb.insert("S")
	rb.insert("T")
	rb.insert("U")
	rb.insert("V")
	rb.insert("W")
	rb.insert("X")
	rb.insert("Y")
	rb.insert("Z")
	rb.insert("A")
	rb.insert("B")
	rb.insert("C")
	rb.insert("D")
	rb.insert("E")
	rb.insert("F")
	rb.insert("G")
	rb.insert("H")
	rb.insert("I")	
	rb.insert("J")
	rb.insert("K")
	
	' Validate tree structure
	rb.validate
	
	Call objLog.Write( "Search for: A" , LOG_INFO )
		
	' Try to find an item will return std_auto wrapper object
	Set auto = rb.find( "A" )
	
	' If the wrapper object is nothing the item doesn't exist
	If auto Is Nothing Then
		Call objLog.Write( "No such item exists" , LOG_INFO )
	Else
		' Actual data is stored in std_auto.data
		Call objLog.Write( auto , LOG_INFO )
	End If
	
	Call objLog.Write( "Search for: ZZ (Should me missing)" , LOG_INFO )
	
	' Try to find an item will return std_auto wrapper object
	Set auto = rb.find( "ZZ" )
	
	' If the wrapper object is nothing the item doesn't exist
	If auto Is Nothing Then
		Call objLog.Write( "No such item exists" , LOG_INFO )
	Else
		' Actual data is stored in std_auto.data
		Call objLog.Write( auto , LOG_INFO )
	End If
	
	' Show current tree size
	Call objLog.Write( "Size of tree: " & rb.Size , LOG_INFO )
	
	Call objLog.Write( "Tree Iterator Test - BFI" , LOG_INFO )
	
	' Get std_iterator
	Set itr = rb.get_iterator()
	
	' If the iterator has_more
	While itr.more_data
		' Get the next std_auto wrapped item
		Set auto = itr.get_next()
		' Get data in the auto type ( we know it's a string so echo it )
		Call objLog.Write( "Tree Iterator: " & auto , LOG_INFO )
	Wend
	
	Call objLog.Write( "Size of tree: " & rb.Size , LOG_INFO )		
	rb.remove("L")
	rb.remove("M")
	rb.remove("N")
	rb.remove("O")
	rb.remove("P")
	rb.remove("Q")
	rb.remove("R")	
	rb.remove("S")
	rb.remove("T")
	rb.remove("U")
	rb.remove("V")
	rb.remove("W")
	rb.remove("X")
	rb.remove("Y")
	rb.remove("Z")
	rb.remove("A")
	
	Call objLog.Write( "Size of tree: " & rb.Size , LOG_INFO )	
	rb.clear
	Call objLog.Write( "Size of tree: " & rb.Size , LOG_INFO )	
	
	Call objLog.Write( "Validate RB Tree Balance" , LOG_INFO )
	
	rb.validate
	
		
'************************************************************************************************
'                                     std_stack tests start
'************************************************************************************************
		
	Dim stacker : Set stacker = New std_stack
	stacker.Push("A")
	stacker.Push("B")
	stacker.Push("C")
	stacker.Push("D")
	stacker.Push("E")
	stacker.Push("F")
	stacker.Push("G")
	stacker.Push("H")
	stacker.Push("I")
	
	Call objLog.Write( "Stack Size: " & stacker.size , LOG_INFO )
	
	Call objLog.Write( "Stack Iterator Test (LIFO)" , LOG_INFO )
	
	' Get std_iterator
	Set itr = stacker.get_iterator
	
	' If the iterator has_more
	While itr.more_data
		' Get the next std_auto wrapped item
		Set auto = itr.get_next()
		' Get data in the auto type ( we know it's a string so echo it )
		Call objLog.Write( "Stack Iterator: " & auto , LOG_INFO )
	Wend
	
	Call objLog.Write( "Stack Size: " & stacker.size , LOG_INFO )
	
	Call objLog.Write( "Stack Pop Test (LIFO)" , LOG_INFO )
	While stacker.Size > 0
		Call objLog.Write( "Stack Peek: " & stacker.peek , LOG_INFO )
		Call objLog.Write( "Stack Deque: " & stacker.pop , LOG_INFO )
	Wend
	
	Call objLog.Write( "Stack Size: " & stacker.size , LOG_INFO )
	
'************************************************************************************************
'                                     std_queue tests start
'************************************************************************************************
	Call objLog.Write( "Test Queue Insert" , LOG_INFO )

	Dim que : Set que = New std_queue
	que.enqueue("A")
	que.enqueue("B")
	que.enqueue("C")
	que.enqueue("D")
	que.enqueue("E")
	que.enqueue("F")
	que.enqueue("G")
	que.enqueue("H")
	que.enqueue("I")
	
	Call objLog.Write( "Queue Size: " & que.size , LOG_INFO )
	
	Call objLog.Write( "Queue Iterator Test (FIFO)" , LOG_INFO )
	
	' Get std_iterator
	Set itr = que.get_iterator
	
	' If the iterator has_more
	While itr.more_data
		' Get the next std_auto wrapped item
		Set auto = itr.get_next()
		' Get data in the auto type ( we know it's a string so echo it )
		Call objLog.Write( "Queue Iterator: " & auto , LOG_INFO )
	Wend
	
	Call objLog.Write( "Queue Size: " & que.size , LOG_INFO )
		
	Call objLog.Write( "Queue Dequeue Test (FIFO)" , LOG_INFO )
	
	While que.has_items
		Call objLog.Write( "Queue Peek: " & que.peek , LOG_INFO )
		Call objLog.Write( "Queue Deque: " & que.dequeue , LOG_INFO )
	Wend
	
	Call objLog.Write( "Queue Size: " & que.size , LOG_INFO )
	
'************************************************************************************************
'                                     std_deque tests start
'************************************************************************************************

	Dim dq : Set dq = New std_deque
	Call objLog.Write( "Deque test push_back" , LOG_INFO )
	dq.push_back("A")
	dq.push_back("B")
	dq.push_back("C")
	dq.push_back("D")
	dq.push_back("E")
	dq.push_back("F")
	dq.push_back("G")
	dq.push_back("H")
	dq.push_back("I")

	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )
	
	Call objLog.Write( "Deque Iterator Test (Forward)" , LOG_INFO )
	
	' Get std_iterator forward iterator
	Set itr = dq.get_iterator
	
	' If the iterator has_more
	While itr.more_data
		' Get the next std_auto wrapped item
		Set auto = itr.get_next()
		' Get data in the auto type ( we know it's a string so echo it )
		Call objLog.Write( "Deque Iterator: " & auto , LOG_INFO )
	Wend
	
	Call objLog.Write( "Deque Iterator Test (Reverse)" , LOG_INFO )
	
	' Get std_iterator forward iterator
	Set itr = dq.get_rev_iterator
	
	' If the iterator has_more
	While itr.more_data
		' Get the next std_auto wrapped item
		Set auto = itr.get_next()
		' Get data in the auto type ( we know it's a string so echo it )
		Call objLog.Write( "Deque Iterator: " & auto , LOG_INFO )
	Wend
	
	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )
		
	Call objLog.Write( "Deque Test peek_front and pop_front" , LOG_INFO )
	
	' Push back \ pop front
	While dq.has_items
		Call objLog.Write( "Deque Peek Front: " & dq.peek_front , LOG_INFO )
		Call objLog.Write( "Deque Pop Front: " & dq.pop_front , LOG_INFO )
	Wend
	
	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )				
				
	Call objLog.Write( "Deque test push_front" , LOG_INFO )
	dq.push_front("A")
	dq.push_front("B")
	dq.push_front("C")
	dq.push_front("D")
	dq.push_front("E")
	dq.push_front("F")
	dq.push_front("G")
	dq.push_front("H")
	dq.push_front("I")
	
	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )	
					
	Call objLog.Write( "Deque Iterator Test (Forward)" , LOG_INFO )
	
	' Get std_iterator forward iterator
	Set itr = dq.get_iterator
	
	' If the iterator has_more
	While itr.more_data
		' Get the next std_auto wrapped item
		Set auto = itr.get_next()
		' Get data in the auto type ( we know it's a string so echo it )
		Call objLog.Write( "Deque Iterator: " & auto , LOG_INFO )
	Wend
	
	Call objLog.Write( "Deque Iterator Test (Reverse)" , LOG_INFO )
	
	' Get std_iterator forward iterator
	Set itr = dq.get_rev_iterator
	
	' If the iterator has_more
	While itr.more_data
		' Get the next std_auto wrapped item
		Set auto = itr.get_next()
		' Get data in the auto type ( we know it's a string so echo it )
		Call objLog.Write( "Deque Iterator: " & auto , LOG_INFO )
	Wend
	
	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )	
	
	Call objLog.Write( "Deque Test peek_back and pop_back" , LOG_INFO )
	
	' Push Front \ Pop Back
	While dq.has_items
		Call objLog.Write( "Deque Peek Back: " & dq.peek_back , LOG_INFO )
		Call objLog.Write( "Deque Pop Back: " & dq.pop_back , LOG_INFO )
	Wend
	
	Call objLog.Write( "Deque Test push_back \ pop_back" , LOG_INFO )
	dq.push_back("A")
	dq.push_back("B")
	dq.push_back("C")
	dq.push_back("D")
	dq.push_back("E")
	dq.push_back("F")
	dq.push_back("G")
	dq.push_back("H")
	dq.push_back("I")
	
	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )
	
	While dq.has_items
		Call objLog.Write( "Deque Peek Back: " & dq.peek_back , LOG_INFO )
		Call objLog.Write( "Deque Pop Back: " & dq.pop_back , LOG_INFO )
	Wend
	
	
	Call objLog.Write( "Deque Test push_front \ pop_front" , LOG_INFO )
	dq.push_front("A")
	dq.push_front("B")
	dq.push_front("C")
	dq.push_front("D")
	dq.push_front("E")
	dq.push_front("F")
	dq.push_front("G")
	dq.push_front("H")
	dq.push_front("I")
	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )
	
	While dq.has_items
		Call objLog.Write( "Deque Peek Front: " & dq.peek_front , LOG_INFO )
		Call objLog.Write( "Deque Pop Front: " & dq.pop_front , LOG_INFO )
	Wend

	Call objLog.Write( "Push Front with random delete" , LOG_INFO )
	dq.push_front("A")
	dq.push_front("B")
	dq.push_front("C")
	dq.push_front("D")
	dq.push_front("E")
	dq.push_front("F")
	dq.push_front("G")
	dq.push_front("H")
	dq.push_front("I")

	While dq.has_items
		If dq.size Mod 2 = 0 Then 
			Call objLog.Write( "Deque Pop Front: " & dq.pop_front , LOG_INFO )
			Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )
		Else
			Call objLog.Write( "Deque Pop Back: " & dq.pop_back , LOG_INFO )
			Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )
		End If
	Wend

	Call objLog.Write( "Deque Size: " & dq.size , LOG_INFO )
	
'************************************************************************************************
'                                   std_fs_search tests start
'************************************************************************************************

	Call objLog.Write( "File Searcher Test - If needle.txt is found force stop search (this will take a while)" , LOG_INFO )
	Dim o_needle ' If we fine needle.txt this will have the path
	Dim s_pref : Set s_pref = New std_searchprefs
	
	' Search criteria (Only use callback if the file or folder matches the following pattern)
	s_pref.SearchExp = "^.*[.]txt$"
	' Enable File Search "FSON" enable Directory Search "DSON"
	' Find only read only files matching search criteria "FRDO"
	' Find any folder which matches the search criteria "DANY"
	s_pref.Flags = FS_ATTR_FSON Or FS_ATTR_FANY Or FS_ATTR_DSON Or FS_ATTR_DANY
	' Search 3 levels deep from search root.
	s_pref.Depth = 1
	
	Dim f_srch : Set f_srch = New std_fs_search
	f_srch.RegisterCallback( GetRef("myFileSearchCallback") )
	If f_srch.Search( "C:\" , s_pref , o_needle ) Then 
		Call objLog.Write( "Search call back issued abort: " & o_needle , LOG_INFO )
	Else
		Call objLog.Write( "Search call back never aborted" , LOG_INFO )
	End If
	
	' Fix NSLookup create an object to deal with this
	Dim strNS , strNSIP , strName , arrIPs	
	If NSLookup( ping_lookup_host , strNS , strNSIP , strName , arrIPs ) Then 
		Call objLog.Write( "NSLookup Success" , LOG_INFO )
		Call objLog.Write( "   Name Server: " & strNS , LOG_INFO )
		Call objLog.Write( "Name Server IP: " & strNSIP , LOG_INFO )
		Call objLog.Write( "          Host: " & strName , LOG_INFO )
		Dim s
		For Each s In arrIPs 
			Call objLog.Write( "    Host IP(s): " & s , LOG_INFO )
		Next
	Else
		Call objLog.Write( "NSLookup Faileed", LOG_ERROR )
	End If
	
	' Ping Function Test
	Call objLog.Write("Ping Test" , LOG_INFO )
	
	Dim bs , br , bl , mi , ma , av
	If Ping(  ping_lookup_host , bs , br , bl , mi , ma , av ) Then 
		Call objLog.Write("Ping passed: " & ping_lookup_host , LOG_INFO )
		Call objLog.Write("Bytes Sent: " & bs , LOG_INFO )
		Call objLog.Write("Bytes Recv: " & br , LOG_INFO )
		Call objLog.Write("Bytes Lost: " & bl , LOG_INFO )
		Call objLog.Write("Min ms: " & mi , LOG_INFO )
		Call objLog.Write("Max ms: " & ma , LOG_INFO )
		Call objLog.Write("Average ms: " & av , LOG_INFO )
	Else
		Call objLog.Write("Ping failed: " & ping_lookup_host , LOG_ERROR )
	End If
	
	
'************************************************************************************************
'                                   std_process tests start
'************************************************************************************************

	Call objLog.Write("Test Process" , LOG_INFO )
	
	Dim p : Set p = New std_process
	Dim str ' Some string value 
	
	If  p.ConnectProvider( "." ) Then 
		Dim proc : proc = "explorer.exe"
		If p.ProcessExists( proc ) Then 
			Call objLog.Write("Process " & proc & " exists" , LOG_INFO )
		Else
			Call objLog.Write("Process " & proc & " does not exist" , LOG_WARNING )
		End If
		
		proc = Array()
		
		If p.ProcessList( proc ) = PROCESS_SUCCESS Then
			Call objLog.Write( "Listing processes..." , LOG_INFO )
			For Each str In proc
				Call objLog.Write( str , LOG_INFO )
			Next
		Else
			Call objLog.Write("Cant list process" , LOG_ERROR )
		End If
		
		proc = "calc.exe"
		
		Call objShell.Run( proc , False ) 
		
		Call WScript.Sleep( 5000 )
		
		If p.ProcessKill( proc ) = PROCESS_SUCCESS Then
			Call objLog.Write("Process Killed: " & proc , LOG_INFO )
		Else
			Call objLog.Write("Process not stopped: " & proc , LOG_WARNING )
		End If
		
	Else
		Call objLog.Write("Process Class - Provider connect failed", LOG_ERROR )
	End If

'TODO finish recusive registy searcher	
'************************************************************************************************
'                                   std_process tests start
'************************************************************************************************
	
	Call objLog.Write("Registry Class - Test", LOG_INFO )
	Dim r : Set r = New std_registry
	If  r.ConnectProvider32( "." ) Then 
		If r.ReadValue( HKEY_LOCAL_MACHINE , REG_SZ , "SYSTEM\CurrentControlSet\Control\Session Manager\Environment" , "PROCESSOR_ARCHITECTURE" , str ) = 0 Then
			Call objLog.Write("Regitry Class - Reg value read: " & str , LOG_INFO )
		Else
			Call objLog.Write("Regitry Class - Failed to read registry value" , LOG_ERROR )
		End If  
	Else
		Call objLog.Write("Regitry Class - Provider connect failed", LOG_ERROR )
	End If 
	
'************************************************************************************************
'                                   ini tests start
'************************************************************************************************

Call objLog.Write("Ini - Test", LOG_INFO )
	
Dim installOptions
installOptions="c:\temp\test.ini"

Dim nn,xx
For nn = 0 To 10
	Call objLog.Write( "SECTION-"&nn , LOG_INFO )
	For xx = 0 To 10
		Call objLog.Write( "VALUE-"&xx , LOG_INFO )
		Call ini_write( "SECTION-"&nn, "VALUE-"&xx , xx , installOptions )
	Next
Next

Dim d
Call ini_read( null , null , "cool" , d , installOptions )

If TypeName(d) = "Variant()" Then 
	Dim n
	For Each n In d
		WScript.Echo "Array: " & n
	Next
Else
	WScript.Echo "Value: " & d
End If 
	
' Close log file
Call objLog.Close()
		
End Function
		